classdef StandardLinearMixedModel
%  This feature is intended for internal use only and is subject to change
%  at any time without warning. 

%StandardLinearMixedModel - An internal utility to fit Linear Mixed Effects
%   (LME) models in standard form. This feature is intended for internal 
%   use only and is subject to change at any time without warning. 
%%
% $y = X \beta + Z b + \varepsilon$
% 
% * $y$ is a $N \times 1$ vector.
% * $X$ is a $N \times p$ fixed effects design matrix
% * $\beta$ is a $p \times 1$ fixed effect vector
% * $Z$ is a $N \times q$ random effects design matrix
% * $b$ is a $q \times 1$ random effect vector
% * $\varepsilon \sim N(0,\sigma^2 I_N)$
% * $b \sim N(0, \sigma^2 D)$
% * $\Psi = \sigma^2 D$
%
%   The purpose of this class is to:
%   (1) Fit LME models in standard form by ML or REML.-
%   (2) Refit the LME model after changing some inputs.-
%   (3) Compute the estimate betaHat of beta.-
%   (4) Compute the estimate thetaHat of theta.-
%   (5) Compute the estimate sigmaHat of sigma.-
%   (6) Compute the estimated BLUP bHat of b.-
%   (7) Estimate covariance of betaHat.-
%   (8) Perform hypothesis tests on beta.-
%   (9) CIs for contrasts of beta. -
%  (10) Compute covariance of [betaHat - beta;bHat - b].-
%  (11) Peform hypothesis tests on [beta;b]. -
%  (12) CIs for contrasts of [beta;b]. -
%  (13) Estimate covariance of [thetaHat;log(sigmaHat)].-
%  (14) Estimate covariance of [etaHat;log(sigmaHat)] where etaHat is the
%       Natural parameter vector for [thetaHat;log(sigmaHat)]. - 
%  (15) Compute confidence intervals on the canonical parameter vector
%       [heta;log(sigma)]. - 
%  (16) Compute conditional/marginal fitted values.-
%  (17) Compute conditional/marginal residuals of various types. -
%  (18) Generate random data from fitted model. -
%  (19) Make predictions on new data. -
%  (20) Compute the posterior covariance of random effects vector. -
%  (21) Compute various goodness of fit criterion. -
%
%   StandardLinearMixedModel methods:
%      StandardLinearMixedModel - Create/Fit a LME model in standard form.
%      refit - Fit/refit a previously initialized LME model.
%      initstats - Make a fitted LME model ready for stats.
%      fitted - Get fitted values.
%      residuals - Convenience method to get various types of residuals.
%      modelCriterion - Compute various goodness of fit criterion.
%      fixedEffects - Summary stats on fixed effects.
%      randomEffects - Summary stats on random effects.
%      covarianceParameters - Summary stats on covariance parameters.
%      betaTTest - Perform T-test on beta.
%      betaFTest - Perform F-test on beta.
%      betaCI - Get confidence intervals on c'*beta.
%      betaBTTest - Perform T-test on [beta;b].
%      betaBFTest - Perform F-test on [beta;b].
%      betaBCI - Get confidence intervals on c'*[beta;b].
%      random - Generate random data from fitted model.
%      predict - Make predictions on new data.
%      predictTable - Summary table of predictions on new data.
%      postCovb - Posterior covariance of random effects.
%      hetaSigmaCI - Confidence intervals on covariance parameters (Canonical scale).    
%      dfBetaTTest - Satterthwaite DF for T-test on beta.
%      dfBetaFTest - Satterthwaite DF for F-test on beta.
%      dfBetaBTTest - Satterthwaite DF for T-test on [beta;b].
%      dfBetaBFTest - Satterthwaite DF for F-test on [beta;b].     
%
%   StandardLinearMixedModel properties:
%      y - Response vector.
%      X - Fixed effects design matrix.
%      Z - Random effects design matrix.
%      Psi - An object of type CovarianceMatrix encapsulating $\sigma^2 D$.
%      FitMethod - Method used to fit the LME.  
%      N - Number of rows in y, X and Z.
%      p - Number of columns in X.       
%      q - Number of columns in Z.
%      rankX - Rank of the fixed effects design matrix X.
%      betaHat - Estimated fixed effects vector.        
%      bHat - Estimated Best Linear Unbiased Predictor of random effects.
%      sigmaHat - Estimated residual standard deviation.       
%      thetaHat - Estimated unconstrained parameter for matrix D.
%      loglikHat - Maximized log likelihood or restricted log likelihood.   
%      covbetaHat - Estimated covariance of betaHat.
%      covthetaHatlogsigmaHat - Estimated covariance on unconstrained scale.       
%      covetaHatlogsigmaHat - Estimated covariance on Natural scale.        
%      covbetaHatbHat - Estimated covariance of [betaHat-beta;bHat-b].
%      isFitToData - Flag to mark a fitted StandardLinearMixedModel object.
%      isReadyForStats - Flag to mark the object ready for doing stats.

%   Copyright 2012-2013 The MathWorks, Inc.    

% Public properties supplied by the user. Checked OK3.
    properties (GetAccess=public, SetAccess=public)
%y - N by 1 response vector used to fit the LME.        
        y

%X - N by p fixed effects design matrix used to fit the LME.        
        X

%Z - N by q random effects design matrix used to fit the LME.        
        Z

%Psi - An object of type CovarianceMatrix representing the matrix Psi =
%   sigma^2 * D(theta). If not using random initialization, optimization
%   will be initialized using the value of unconstrained parameters
%   contained in this object. The initial value of sigma in Psi will be
%   ignored. When optimization completes, Psi will contain the optimal
%   theta and sigma.
        Psi
        
%FitMethod - The method used to fit the LME model - either 'ML' for maximum 
%   likelihood or 'REML' for restricted maximum likelihood.
        FitMethod                    
    end

% Internal constants. Checked OK3.
    properties (Constant=true, Hidden=true)       
%AllowedFitMethods - The fitting methods that we currently support.        
        AllowedFitMethods = {'ML','REML'};
        
%AllowedDFMethods - The DF methods that we currently support.        
        AllowedDFMethods = {'None','Residual','Satterthwaite'};
        
%AllowedResidualTypes - The Residual types that we currently support.
        AllowedResidualTypes = {'Raw','Pearson','Standardized'};
        
%AllowedOptimizers - A list of allowed optimizers.
        AllowedOptimizers = {'fminunc','quasinewton'};
        
%AllowedInitializationMethods - A list of allowed initialization methods.
        AllowedInitializationMethods = {'random','default'};
    end
    
% Public properties storing size information. Checked OK3.    
    properties (GetAccess=public, SetAccess=protected)
%N - Number of observations. This is the number of rows in y, X and Z.
        N
        
%p - Number of fixed effects. This is the number of columns in X. 
        p
        
%q - Number of random effects. This is the number of columns in Z.
        q        
        
%rankX - Rank of the fixed effects design matrix.
        rankX
    end
  
% Public properties storing the name of the optimization algorithm to use, 
% its optimization options and the type of initialization to use. 
    properties(GetAccess=public, SetAccess=protected)

%Optimizer - A string containing the name of the optimizer.        
        Optimizer = 'quasinewton';
        
%OptimizerOptions - A structure containing the optimizer options.        
        OptimizerOptions = struct([]);
        
%InitializationMethod - A string containing the method used to compute the 
%   initial value of theta to start the optimization.        
        InitializationMethod = 'default';

%CheckHessian - A logical scalar indicating whether to perform the Hessian 
%   checks in initstats.    
        CheckHessian = false;
        
    end
    
% Public properties for estimated LME model parameters. Checked OK3.   
    properties (GetAccess=public, SetAccess=protected)
%betaHat - Estimated fixed effects vector.
        betaHat
        
%bHat - Estimated Best Linear Unbiased Predictor (EBLUP) of random effects 
%   vector.
        bHat
        
%sigmaHat - Estimated residual standard deviation.
        sigmaHat
        
%thetaHat - Estimated vector of unconstrained parameters for matrix D.
        thetaHat        
        
%loglikHat - Maximized log likelihood (if FitMethod = 'ML') or maximized 
%   restricted log likelihood (if FitMethod = 'REML').        
        loglikHat                    
    end
        
% Public properties for stats info on LME model parameters. Checked OK3.
    properties (GetAccess=public, SetAccess=protected)        
%covbetaHat - Estimated covariance of betaHat.        
        covbetaHat
                
%covthetaHatlogsigmaHat - Estimated covariance of [thetaHat;log(sigmaHat)].        
        covthetaHatlogsigmaHat      
        
%covetaHatlogsigmaHat - Estimated covariance of [etaHat;log(sigmaHat)]. 
%   etaHat is the Natural parameter vector corresponding to thetaHat.        
        covetaHatlogsigmaHat
        
%covbetaHatbHat - Estimated covariance of [betaHat-beta;bHat-b].
        covbetaHatbHat
    end        
    
% Public properties storing state information. Checked OK3.
    properties (GetAccess=public, SetAccess=protected)
%isFitToData - True if this is a fitted StandardLinearMixedModel object.
        isFitToData = false;
        
%isReadyForStats - True if this StandardLinearMixedModel object is ready 
%   for stats.
        isReadyForStats = false;
    end
    
% Set methods for X, y, Z, Psi and FitMethod. Checked OK3.    
    methods
        
        function slme = set.y(slme,newy)
        
            if ~isempty(slme.y)
                % (1) Validate newy.
                newy = validatey(slme,newy);
                
                % (2) Invalidate the fit.
                slme = invalidateFit(slme);
            end
            
            % (3) Set the new y.
            slme.y = newy;
            
        end % end of set.y
        
        function slme = set.X(slme,newX)
            
            if ~isempty(slme.X)
                % (1) Validate newX.
                newX = validateX(slme,newX);
                
                % (2) Invalidate the fit.
                slme = invalidateFit(slme);
            end
            
            % (3) Set the new X.
            slme.X = newX;
            
        end % end of set.X
        
        function slme = set.Z(slme,newZ)
            
            if ~isempty(slme.Z)
                % (1) Validate newZ.
                newZ = validateZ(slme,newZ);
                
                % (2) Invalidate the fit.
                slme = invalidateFit(slme);
            end            
            
            % (3) Set the new Z.
            slme.Z = newZ;
            
        end % end of set.Z
        
        function slme = set.Psi(slme,newPsi)
            
            if ~isempty(slme.Psi)
                % (1) Validate newPsi.
                newPsi = validatePsi(slme,newPsi);
                
                % (2) Invalidate the fit.
                slme = invalidateFit(slme);
            end
            
            % (3) Set the new Psi.
            slme.Psi = newPsi;
            
        end % end of set.Psi
        
        function slme = set.FitMethod(slme,newFitMethod)
            
            if ~isempty(slme.FitMethod)
                % (1) Validate newFitMethod.
                newFitMethod = validateFitMethod(slme,newFitMethod);
                
                % (2) Invalidate the fit.
                slme = invalidateFit(slme);
            end
            
            % (3) Set the new FitMethod.
            slme.FitMethod = newFitMethod;            
            
        end % end of set.FitMethod
        
    end
    
% Private methods for input validation. Checked OK3.
    methods (Access=private)               
        
        function [X,y,Z,Psi,FitMethod] = validateInputs(slme,X,y,Z,Psi,FitMethod)             %#ok<INUSL>
%validateInputs - Basic checks on inputs. Return the validated inputs.
%   [X,y,Z,Psi,FitMethod] = validateInputs(slme,X,y,Z,Psi,FitMethod) takes
%   user supplied inputs, validates them and returns the validated results.

            % Assert that:
            % (1) X is a numeric, real, matrix of size N by p.
            [N,p] = size(X);
            % <entry key="BadX">Fixed Effects design matrix X must be a numeric real matrix.</entry>
            assertThat(isnumeric(X) & isreal(X) & ismatrix(X),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadX');
            
            % (2) X must not have any NaN or Inf values.
            % <entry key="NoNaNInfAllowed">NaN or Inf values are not allowed in {0}.</entry>  
            assertThat(~hasNaNInf(X),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:NoNaNInfAllowed','X');
            
            % (3) X has rank p.
            % <entry key="MustBeFullRank_X">Fixed Effects design matrix X must be of full column rank.</entry>            
            assertThat(rank(X) == p,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:MustBeFullRank_X');                        
            
            % (4) y is a numeric, real, vector of size N by 1.
            % <entry key="BadY">Response vector y must be a numeric real vector with {0} rows.</entry>
            assertThat(isnumeric(y) & isreal(y) & isvector(y),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadY',num2str(N)); 
            assertThat(       all(size(y) == [N,1])          ,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadY',num2str(N)); 

            % (5) y must not have any NaN or Inf values.
            % <entry key="NoNaNInfAllowed">NaN or Inf values are not allowed in {0}.</entry>
            assertThat(~hasNaNInf(y),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:NoNaNInfAllowed','y');
            
            % (6) Z is a numeric, real, matrix of size N by q.
            q = size(Z,2);
            % <entry key="BadZ">Random Effects design matrix Z must be a numeric real matrix with {0} rows.</entry>
            assertThat( isnumeric(Z) & isreal(Z) & ismatrix(Z), 'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadZ',num2str(N));
            assertThat(        all(size(Z) == [N,q])          , 'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadZ',num2str(N));

            % (7) Z must not have any NaN or Inf values.
            % % <entry key="NoNaNInfAllowed">NaN or Inf values are not allowed in {0}.</entry>  
            assertThat(~hasNaNInf(Z),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:NoNaNInfAllowed','Z');   
            
            % (8) Ensure that Z is sparse.
            if ~issparse(Z)
                Z = sparse(Z);
            end                                 
            
            % (9) Psi is an object of type CovarianceMatrix.
            % <entry key="BadPsi">Psi must be a subclass of CovarianceMatrix.</entry>
            assertThat(isa(Psi,'classreg.regr.lmeutils.covmats.CovarianceMatrix'),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadPsi');
            
            % (10) Psi.Size is q.
            % <entry key="BadPsi_Size">Psi must be of size {0}.</entry>
            assertThat(Psi.Size == q,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadPsi_Size',num2str(q));
            
            % (11) FitMethod is a string containing either 'ML' or 'REML'.
            FitMethod = internal.stats.getParamVal(FitMethod,classreg.regr.lmeutils.StandardLinearMixedModel.AllowedFitMethods,'FitMethod');
            
            % (12) N is >= 2. p or q may be 0.
            % <entry key="BadN">Number of observations N must be >= 2.</entry>
            assertThat(N >= 2,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadN');
            
            % (13) N >= p.
            % <entry key="BadN_MinN">Number of observations N must be >= {0} to fit this data.</entry>
            assertThat(N >= p,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadN_MinN',num2str(p));             
            
        end % end of validateInputs.                      
        
        function slme = invalidateFit(slme)
%invalidateFit - Invalidate the possibly fitted StandardLinearMixedModel.
%   slme = invalidateFit(slme) marks the object as not fitted to data and
%   not ready for stats and returns the modified object.

            % (1) Invalidate the fit.
            slme.isFitToData = false;
            slme.isReadyForStats = false;
            
        end % end of invalidateFit.
        
        function X = validateX(slme,X)
        
            % (1) Ensure X is a numeric real matrix of size slme.N by 
            % slme.p. 
            % <entry key="InValidX_Size">X must be a numeric real matrix with {0} rows and {1} columns.</entry>
            assertThat(isnumeric(X) & isreal(X) & ismatrix(X),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:InValidX_Size',num2str(slme.N),num2str(slme.p)); 
            assertThat(    all(size(X) == [slme.N,slme.p])   ,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:InValidX_Size',num2str(slme.N),num2str(slme.p)); 
            
            % (2) Ensure no NaN/Inf values in X.
            % <entry key="NoNaNInfAllowed">NaN or Inf values are not allowed in {0}.</entry>
            assertThat(~hasNaNInf(X),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:NoNaNInfAllowed','X');
            
            % (3) Ensure that rank of X is equal to slme.p.
            % <entry key="InValidX_Rank">X must have rank {0}.</entry>  
            assertThat(rank(X) == slme.p,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:InValidX_Rank',num2str(slme.p));                        
            
        end % end of validateX.
        
        function y = validatey(slme,y)
            
            % (1) Ensure y is a numeric, real, vector of size slme.N by 1.
            % <entry key="InValidY_Size">y must be a numeric real vector with {0} rows and {1} column.</entry>
            assertThat(isnumeric(y) & isreal(y) & isvector(y),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:InValidY_Size',num2str(slme.N),num2str(1)); 
            assertThat(     all(size(y) == [slme.N,1])       ,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:InValidY_Size',num2str(slme.N),num2str(1)); 

            % (2) Ensure no NaN or Inf values in y.
            % <entry key="NoNaNInfAllowed">NaN or Inf values are not allowed in {0}.</entry>  
            assertThat(~hasNaNInf(y),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:NoNaNInfAllowed','y');
            
        end % end of validatey.
        
        function Z = validateZ(slme,Z)
            
            % (1) Ensure Z is a numeric, real matrix of size slme.N by
            % slme.q.
            % <entry key="InValidZ_Size">Z must be a numeric real matrix with {0} rows and {1} columns.</entry>
            assertThat(isnumeric(Z) & isreal(Z) & ismatrix(Z),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:InValidZ_Size',num2str(slme.N),num2str(slme.q));
            assertThat(    all(size(Z) == [slme.N,slme.q])   ,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:InValidZ_Size',num2str(slme.N),num2str(slme.q));

            % (2) Ensure no NaN/Inf values in Z.
            % <entry key="NoNaNInfAllowed">NaN or Inf values are not allowed in {0}.</entry>  
            assertThat(~hasNaNInf(Z),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:NoNaNInfAllowed','Z');
            
            % (3) Ensure that Z is sparse.
            if ~issparse(Z)
                Z = sparse(Z);
            end                       
            
        end % end of validateZ.
        
        function Psi = validatePsi(slme,Psi)
           
            % (1) Ensure Psi is an object of type CovarianceMatrix.
            % <entry key="BadPsi">Psi must be a subclass of CovarianceMatrix.</entry>  
            assertThat(isa(Psi,'classreg.regr.lmeutils.covmats.CovarianceMatrix'),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadPsi');
            
            % (2) Ensure Psi.Size is slme.q.
            % <entry key="BadPsi_Size">Psi must be of size {0}.</entry>        
            assertThat(Psi.Size == slme.q,'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadPsi_Size',num2str(slme.q));            
            
        end % end of validatePsi.
        
        function FitMethod = validateFitMethod(slme,FitMethod) %#ok<INUSL>
            
            % (1) FitMethod is a string containing either 'ML' or 'REML'.
            FitMethod = internal.stats.getParamVal(FitMethod,classreg.regr.lmeutils.StandardLinearMixedModel.AllowedFitMethods,'FitMethod');
            
        end % end of validateFitMethod.
        
        function theta = validateTheta(slme,theta)

            % (1) Ensure theta is a numeric, real vector.
            assert( isnumeric(theta) & isreal(theta) & isvector(theta) );
            
            % (2) Ensure that theta has the right size.
            assert( all(size(theta) == [slme.Psi.NumParametersExcludingSigma,1]) );
            
            % (3) Ensure no NaN or Inf values in theta.
            assert( ~hasNaNInf(theta) );
            
        end % end of validateTheta.
        
        function beta = validateBeta(slme,beta)

            % (1) Ensure that beta is a numeric, real vector.
            assert( isnumeric(beta) & isreal(beta) & isvector(beta) );
            
            % (2) Ensure that beta has the right size.
            assert( all(size(beta) == [slme.p,1]) );
            
            % (3) Ensure no NaN/Inf values in beta.
            assert( ~hasNaNInf(beta) );
            
        end % end of validateBeta.
        
        function sigma = validateSigma(slme,sigma) %#ok<INUSL>

            % (1) Ensure that sigma is a numeric, real scalar.
            assert( isnumeric(sigma) & isreal(sigma) & isscalar(sigma) );
            
            % (2) Ensure that sigma is positive.
            assert( sigma > 0 );
            
            % (3) Ensure no NaN/Inf values in sigma.
            assert( ~hasNaNInf(sigma) );
            
        end % end of validateSigma.
                
        function [optimizer,optimizeroptions,initializationmethod] ...
                    = validateOptimizationOptions(slme,optimizer,optimizeroptions,initializationmethod) %#ok<INUSL>
                
            % (1) optimizer must be a string from the list
            % AllowedOptimizers.
            optimizer = internal.stats.getParamVal(optimizer,classreg.regr.lmeutils.StandardLinearMixedModel.AllowedOptimizers,'Optimizer');                     
            
            % (2) initializationmethod must be a string from the list 
            % AllowedInitializationMethods. 
            initializationmethod = internal.stats.getParamVal(initializationmethod,classreg.regr.lmeutils.StandardLinearMixedModel.AllowedInitializationMethods,'InitializationMethod');
                            
            % (3) optimizeroptions must be a struct or a
            % optim.options.SolverOptions object.
            % <entry key="BadOptimizerOptions">''OptimizerOptions'' must be created using statset for ''quasinewton'' and using optimoptions for 'fminunc'.</entry>
            assertThat(isstruct(optimizeroptions) || isa(optimizeroptions,'optim.options.SolverOptions'),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadOptimizerOptions');            
            switch lower(optimizer)
                case 'quasinewton'
                    % (1) optimizeroptions can be empty or a struct.
                    % <entry key="BadOptimizerOptions_quasinewton">''OptimizerOptions'' must be created using statset for ''quasinewton''.</entry>
                    assertThat(isempty(optimizeroptions) || isstruct(optimizeroptions),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadOptimizerOptions_quasinewton');                                                            
                case {'fminunc'}
                    % (1) optimizeroptions can be empty or of class
                    % optim.options.SolverOptions.
                    % <entry key="BadOptimizerOptions_fminunc">''OptimizerOptions'' must be created using optimoptions for 'fminunc'.</entry>
                    assertThat(isempty(optimizeroptions) || isa(optimizeroptions,'optim.options.SolverOptions'),'stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadOptimizerOptions_fminunc');                                        
            end
            
        end % end of validateOptimizationOptions.
        
    end
    
% Private methods related to fitting. Checked OK2.
    methods (Access=private)
        
        function fun = makeObjectiveFunctionForMinimization(slme)
%makeObjectiveFunctionForMinimization - Get objective function to minimize.
%   fun = makeObjectiveFunctionForMinimization(slme) takes an object slme
%   of type StandardLinearMixedModel and depending on slme.FitMethod makes
%   either the objective function for ML or REML.

            % (1) Make an objective function for minimization. 
            %     (a) If slme.FitMethod is 'ML' this is the negative 
            %     profiled log likelihood (beta and sigma profiled out).
            %
            %     (b) If slme.FitMethod is 'REML' this is the negative 
            %     profiled restricted log likelihood (sigma profiled out).
            switch lower(slme.FitMethod)
                case 'ml'
                    fun = makeNegBetaSigmaProfiledLogLikelihoodAsFunctionOfTheta(slme);
                case 'reml'
                    fun = makeNegSigmaProfiledRestrictedLogLikelihoodAsFunctionOfTheta(slme);
                otherwise 
                    % <entry key="BadFitMethod">''FitMethod'' must be either ''ML'' or ''REML''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadFitMethod'));
            end
            
        end % makeObjectiveFunctionForMinimization.
        
        function theta0 = initializeTheta(slme)
%initializeTheta - Return an initial value of theta to start optimization.            
            
            % If initialization method is:
            %   (a) 'default' - then extract theta0 from slme.Psi
            %   (b) 'random'  - then generate theta0 from a standard Normal            
            switch lower(slme.InitializationMethod)
                case 'default'
                    % Use the value stored in slme.Psi.
                    theta0 = getUnconstrainedParameters(slme.Psi);
                case 'random'
                    % Use a random value from N(0,1).
                    theta0 = getUnconstrainedParameters(slme.Psi);
                    theta0 = randn(length(theta0),1);
                otherwise
                    % <entry key="BadInitializationMethod">''InitializationMethod'' must be either ''Default'' or ''Random''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadInitializationMethod'));
            end
                        
        end % end of initializeTheta.
        
        function warnAboutPerfectFit(slme,theta0)
%warnAboutPerfectFit - Warn if theta0 gives a nearly perfect fit.            
%   warnAboutPerfectFit(slme,theta0) takes a StandardLinearMixedModel 
%   object slme and an initial value of the unconstrained parameter vector
%   theta0 and warns the user if we have a nearly perfect fit using theta0.
%   We simply solve the mixed model equations at theta0 and if the
%   estimated residual standard deviation is "small", we display a warning
%   message.

            % (1) Solve mixed model equations at theta0.
            [~,~,sigmaHat,~,~,~] = solveMixedModelEquations(slme,theta0);

            % (2) Warn about perfect fit if sigmaHat is "small".
            sigmaHatTol = sqrt(eps(class(sigmaHat)));
            if abs(sigmaHat) <= sigmaHatTol*std(slme.y)
                % <entry key="Message_PerfectFit">Initial estimate of residual noise standard deviation is almost zero. This may indicate a nearly perfect fit to data.</entry>
                warning(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_PerfectFit'));
            end
            
        end % end of warnAboutPerfectFit.
        
        function [thetaHat,cause] = doMinimization(slme,fun,theta0)
%doMinimization - Minimizes fun starting at theta0.
%   [thetaHat,cause] = doMinimization(slme,fun,theta0) attempts to minimize
%   fun starting at theta0 using the optimizer and optimizer options stored
%   in slme. This is a helper method for solveForThetaHat. The output
%   thetaHat is a best guess for the solution and cause is an integer that 
%   indicates the qualitative reason for termination of the optimizer. 
%   Here's what the integer codes in cause mean:
%
%   cause = 0      means that the problem was successfully solved. In most 
%                  cases this means that magnitude of the gradient is small 
%                  enough. The interpretation is "Local minimum found"
%                  (subject to Hessian checks).
%
%   cause = 1      means that problem may have been successfully solved. In
%                  most cases this means that the step size most recently 
%                  taken was small or the changes in objective function was
%                  small. The interpretation is "Local minimum possible".
%
%   cause = 2      means unable to converge to a solution. This may mean 
%                  that iteration/function evaluation limit was reached or 
%                  unable to converge to the specified tolerances.            

            switch lower(slme.Optimizer)                
                case 'quasinewton'                
                    % (1) Call the optimizer. The sequence of outputs is
                    % [theta,funtheta,gradfuntheta,cause].
                    [thetaHat,~,~,cause] = ...
                        fminqn(fun,theta0,'Options',slme.OptimizerOptions);
                    
                    % (2) For fminqn, cause already has the right meaning.
                    % So, no need for any translation.
                    
                case 'fminunc'                    
                    
                    % (1) Turn optim:fminunc:SwitchingMethod warning off.
                    % Since our default fminunc optimization algorithm is
                    % 'quasi-newton', we do not need to disable the 
                    % switching method warning. However, if we decide to
                    % make fminunc's 'trust-region' algorithm the default
                    % then we would need the next 3 lines.
                    %warnState = warning('query','all');
                    %warning('off','optim:fminunc:SwitchingMethod');
                    %cleanupObj = onCleanup(@() warning(warnState));
                    
                    % (2) Call the optimizer. The sequence of outputs is
                    % [X,FVAL,EXITFLAG].
                    [thetaHat,~,exitflag] = ...
                        fminunc(fun,theta0,slme.OptimizerOptions);
                    
                    % (3) Translate exitflag into the cause interpretation.
                    switch exitflag
                        case 1 
                            % Magnitude of gradient is small enough.
                            cause = 0;
                        case {2,3,5}
                            % Step of function tolerance reached.
                            cause = 1;
                        otherwise
                            % Unable to converge.
                            cause = 2;
                    end
                    
                otherwise
                    % <entry key="BadOptimizerName">''Optimizer'' must be either ''quasinewton'' or ''fminunc''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadOptimizerName'));
            end
            
        end % end of doMinimization.
        
        function thetaHat = solveForThetaHat(slme)
%solveForThetaHat - Find the optimal value of theta and return it.
%   thetaHat = solveForThetaHat(slme) takes a StandardLinearMixedModel 
%   object slme, solves for the optimal theta and returns it. theta is the
%   unconstrained vector that parameterizes the matrix D.

            % (1) Make an objective function for minimization. 
            fun = makeObjectiveFunctionForMinimization(slme);                      
            
            % (2) Get theta0, the initial value of theta.
            theta0 = initializeTheta(slme);
                    
            % (3) Warn about perfect fit at theta0.
            warnAboutPerfectFit(slme,theta0);
            
            % (4) Minimize fun, starting at theta0. Use the optimization
            % routine and optimization options stored in object slme.            
            [thetaHat,cause] = doMinimization(slme,fun,theta0);
            
            % (5) If cause = 0 or 1 then we are good, otherwise display a
            % message that we did not converge.
            if ( cause ~= 0 && cause ~= 1 )
                % <entry key="Message_OptimizerUnableToConverge">Optimizer {0} was unable to converge to a solution.</entry>
                warning(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_OptimizerUnableToConverge',slme.Optimizer));                         
            end
            
        end % end of solveForThetaHat.                                        
        
        function H = objectiveFunctionHessianAtThetaHat(slme)
%objectiveFunctionHessianAtThetaHat - Objective function Hessian.           
%   H = objectiveFunctionHessianAtThetaHat(slme) takes a fitted object slme
%   of type StandardLinearMixedModel and gets the Hessian of the objective
%   function used for minimization evaluated at the optimal slme.thetaHat.

            % (1) Make an objective function for minimization.
            fun = makeObjectiveFunctionForMinimization(slme);

            % (2) Get the Hessian of fun at slme.thetaHat.
            wantRegularized = false;
            H = getHessian(fun,slme.thetaHat,wantRegularized);         

        end % end of objectiveFunctionHessianAtThetaHat.
        
        function checkObjectiveFunctionHessianAtThetaHat(slme)
%checkObjectiveFunctionHessianAtThetaHat - Positive definiteness check.
%   checkObjectiveFunctionHessianAtThetaHat(slme) takes a fitted object
%   slme of type StandardLinearMixedModel and checks if the Hessian H of 
%   the objective function used for minimization evaluated at the optimal 
%   slme.thetaHat is positive definite or not. If H is positive definite 
%   then all is well, otherwise a warning message is printed to the screen.

            % (1) Get Hessian of objective function for minimization at
            % slme.thetaHat.
            H = objectiveFunctionHessianAtThetaHat(slme);
            
            % (2) Get the warning message string if H is not positive
            % definite.
            switch lower(slme.FitMethod)                    
                case 'ml'
                    % <entry key="Message_NotSPDHessian_ML">The Hessian of negative profiled log likelihood as a function of unconstrained parameters is not positive definite at convergence.</entry>
                    msgID = 'stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_NotSPDHessian_ML';
                case 'reml'
                    % <entry key="Message_NotSPDHessian_REML">The Hessian of negative profiled restricted log likelihood as a function of unconstrained parameters is not positive definite at convergence.</entry>   
                    msgID = 'stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_NotSPDHessian_REML';
                otherwise
                    % <entry key="BadFitMethod">''FitMethod'' must be either ''ML'' or ''REML''.</entry>      
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadFitMethod'));
            end
            
            % (3) Ensure H is positive definite. If not, display the
            % message msg created above.            
            checkPositiveDefinite(H,msgID);
            
        end % end of checkObjectiveFunctionHessianAtThetaHat.        
        
        function [betaHat,bHat,sigmaHat,r2,R,R1] = solveMixedModelEquations(slme,theta)
%solveMixedModelEquations - Solve mixed model equations.
%   [betaHat,bHat,sigmaHat,r2,R,R1] = solveMixedModelEquations(slme,theta)
%   takes a StandardLinearMixedModel object slme and solves the mixed model
%   equations at the specified value of theta. First, we solve mixed model 
%   equations using theta to get betaHat and bHat. Then we get sigmaHat 
%   (for ML or REML), r2, R and R1. The meaning of r2, R and R1 is defined
%   in internal LME specs.
    
            % (1) Get X, y, Z, N, p and q from slme.
            X = slme.X;
            y = slme.y;
            Z = slme.Z;
            N = slme.N;
            p = slme.p;
            q = slme.q;

            % (2) Get slme.Psi, set current theta and set sigma = 1.
            Psi = slme.Psi;
            Psi = setUnconstrainedParameters(Psi,theta);
            Psi = setSigma(Psi,1);

            % (3) Get lower triangular Cholesky factor of D matrix in Psi.
            % Lambda has size q by q where q = size(Z,2).
            Lambda = getLowerTriangularCholeskyFactor(Psi);

            % (4) Compute the U matrix. U is N by q.
            U = Z*Lambda;
            
            % TODO: Try to make steps 5 onward more efficient.
            % (5) Get R and S such that S*R'*R*S' = U'*U + eye(q) where R
            % and S are q by q. S is a permutation matrix: S*S' = eye(q).
            Iq = spdiags(ones(q,1),0,q,q);
            [R,status,S] = chol(U'*U + Iq);
 
            % (6) Make sure status is zero.
            if (status ~= 0)
                % <entry key="ErrorSparseCholesky">An error occured during sparse Cholesky factorization.</entry>
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:ErrorSparseCholesky'));
            end
            
            % For steps (7) onwards, we may get a rank deficient X or a
            % singular factor R1. Hence we disable the nearly singular
            % matrix warning and then enable it at the end of this method
            % or on error.
            warnState = warning('query','all');
            warning('off','MATLAB:nearlySingularMatrix');
            warning('off','MATLAB:illConditionedMatrix');
            warning('off','MATLAB:singularMatrix');
            cleanupObj = onCleanup(@() warning(warnState));
            
            % (7) Compute P1 (q by q), Q1 (p by q) and R1 (p by p).
            %P1 = S*R';
            Q1 = ((X'*U)*S) / R;

            try
                R1 = chol(X'*X - Q1*Q1','lower');
            catch ME %#ok<NASGU>
                % We know from theory that R1 must exist.
                R1 = classreg.regr.lmeutils.covmats.CovarianceMatrix.singularLowerChol(X'*X - Q1*Q1');
            end
                        
            % (8) Compute cDeltab (q by 1) and cbeta (p by 1).
            %cDeltab = P1 \ (U'*y);
            cDeltab = R' \ (S'*(U'*y));
            cbeta = R1 \ (X'*y - Q1*cDeltab);
            
            % (9) Compute betaHat and Deltab.
            betaHat = R1' \ cbeta;
            %Deltab = P1' \ (cDeltab - Q1'*betaHat);
            Deltab = S*(R \ (cDeltab - Q1'*betaHat));

            % (10) Compute bHat.
            bHat = Lambda * Deltab;
            
            % (11) Compute r2.
            r2 = sum(Deltab.^2) + sum((y - X*betaHat - Z*bHat).^2);
            
            % (12) Compute sigmaHat.
            switch lower(slme.FitMethod)
                case 'ml'
                    sigma2 = r2/N;
                    sigmaHat = sqrt(sigma2);
                case 'reml'
                    sigma2 = r2/(N - p);
                    sigmaHat = sqrt(sigma2);
                otherwise
                    % <entry key="BadFitMethod">''FitMethod'' must be either ''ML'' or ''REML''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadFitMethod'));
            end
            
            % NOTE: Restore the warning state to that saved in warnState.
            % This is automatically done by the cleanupObj going out of
            % scope. warning(warnState);
            
        end % solveMixedModelEquations.   
        
    end

% Private methods to "make" the various likelihood functions. Checked OK2.
    methods (Access=private)
        
        function fun = makeNegBetaSigmaProfiledLogLikelihoodAsFunctionOfTheta(slme)
%makeNegBetaSigmaProfiledLogLikelihoodAsFunctionOfTheta - This returns the
%   negative profiled log likelihood with beta and sigma profiled out as a
%   function of theta.

            fun = @f0;            
            function y0 = f0(theta)                
                 L = BetaSigmaProfiledLogLikelihood(slme,theta);
                y0 = -1*L;
                % Don't let y0 be equal to -Inf.
                y0 = max(-realmax,y0);
            end
            
        end % end of makeNegBetaSigmaProfiledLogLikelihoodAsFunctionOfTheta.
       
        function fun = makeNegSigmaProfiledRestrictedLogLikelihoodAsFunctionOfTheta(slme)
%makeNegSigmaProfiledRestrictedLogLikelihoodAsFunctionOfTheta - This
%   returns the negative profiled restricted log likelihood with sigma
%   profiled out as a function of theta.

            fun = @f1;
            function y1 = f1(theta)
                 L = SigmaProfiledRestrictedLogLikelihood(slme,theta);
                y1 = -1*L;
                % Don't let y1 be equal to -Inf.
                y1 = max(-realmax,y1);
            end
            
        end % end of makeNegSigmaProfiledRestrictedLogLikelihoodAsFunctionOfTheta.
        
        function fun = makeNegBetaProfiledLogLikelihoodAsFunctionOfThetaLogSigma(slme)
%makeNegBetaProfiledLogLikelihoodAsFunctionOfThetaLogSigma - This returns
%   the negative profiled log likelihood with beta profiled out as a
%   function of x = [theta;log(sigma)].
           
            fun = @f2;
            function y2 = f2(x)
                % (1) Extract theta and logsigma from x.
                   theta = x(1:end-1);
                logsigma = x(end);
                % (2) Evaluate BetaProfiledLogLikelihood at (theta,sigma)
                % and negate the result.
                   sigma = exp(logsigma);
                 L = BetaProfiledLogLikelihood(slme,theta,sigma);
                y2 = -1*L;
            end
            
        end % end of makeNegBetaProfiledLogLikelihoodAsFunctionOfThetaLogSigma.
        
        function fun = makeNegRestrictedLogLikelihoodAsFunctionOfThetaLogSigma(slme)
%makeNegRestrictedLogLikelihoodAsFunctionOfThetaLogSigma - This returns the
%   negative restricted log likelihood as a function of x =
%   [theta;log(sigma)].
            
            fun = @f3;
            function y3 = f3(x)
                % (1) Extract theta and logsigma from x.
                   theta = x(1:end-1);
                logsigma = x(end);
                % (2) Evaluate RestrictedLogLikelihood at (theta,sigma) and
                % negate the result.
                   sigma = exp(logsigma);
                   L = RestrictedLogLikelihood(slme,theta,sigma);
                  y3 = -1*L;                
            end
            
        end % end of makeNegRestrictedLogLikelihoodAsFunctionOfThetaLogSigma.
        
        function fun = makeNegBetaProfiledLogLikelihoodAsFunctionOfEtaLogSigma(slme)
%makeNegBetaProfiledLogLikelihoodAsFunctionOfEtaLogSigma - This returns
%   the negative profiled log likelihood with beta profiled out as a
%   function of x = [eta;log(sigma)] where eta is the Natural parameter
%   vector.

            fun = @f6;
            function y6 = f6(x)
                % (1) Extract eta, logsigma and sigma from x.
                     eta = x(1:end-1);
                logsigma = x(end);
                   sigma = exp(logsigma);
                % (2) Set sigma and eta in slme.Psi.
                Psi = slme.Psi;
                Psi = setSigma(Psi,sigma);
                Psi = setNaturalParameters(Psi,eta);
                % (3) Get theta from Psi.
                theta = getUnconstrainedParameters(Psi);
                % (4) Evaluate BetaProfiledLogLikelihood at (theta,sigma)
                % and negate the result.
                L = BetaProfiledLogLikelihood(slme,theta,sigma);
                y6 = -1*L;                
            end

        end % end of makeNegBetaProfiledLogLikelihoodAsFunctionOfEtaLogSigma.
        
        function fun = makeNegRestrictedLogLikelihoodAsFunctionOfEtaLogSigma(slme)
%makeNegRestrictedLogLikelihoodAsFunctionOfEtaLogSigma - This returns the
%   negative restricted log likelihood as a function of x =
%   [eta;log(sigma)] where eta is the Natural parameter vector.
            
            fun = @f7;
            function y7 = f7(x)
                % (1) Extract eta, logsigma and sigma from x.
                     eta = x(1:end-1);
                logsigma = x(end);
                   sigma = exp(logsigma);
                % (2) Set sigma and eta in slme.Psi.
                Psi = slme.Psi;
                Psi = setSigma(Psi,sigma);
                Psi = setNaturalParameters(Psi,eta);
                % (3) Get theta from Psi.
                theta = getUnconstrainedParameters(Psi);
                % (4) Evaluate RestrictedLogLikelihood at (theta,sigma) and
                % negate the result.
                L = RestrictedLogLikelihood(slme,theta,sigma);
                y7 = -1*L;
            end

            
        end % end of makeNegRestrictedLogLikelihoodAsFunctionOfEtaLogSigma.
    end
    
% Public methods related to fitting. Checked OK3.   
    methods (Access=public)                           
        
        function slme = StandardLinearMixedModel(X,y,Z,Psi,FitMethod,dofit,dostats,varargin)
%

%StandardLinearMixedModel - Create a LME model in standard form.
%   slme = StandardLinearMixedModel(X,y,Z,Psi,FitMethod,dofit,dostats)
%   takes the N by p fixed effects design matrix X, N by 1 response vector
%   y, N by q random effects design matrix Z, an object Psi of type
%   CovarianceMatrix encapsulating the random effects covariance matrix
%   (sigma^2 * D), a string FitMethod (either 'ML' or 'REML') and returns a
%   fitted StandardLinearMixedModel object slme if dofit is true. The
%   residual error is assumed to have an isotropic diagonal covariance with
%   variance sigma^2. If dofit is false then an unfitted object is returned
%   which you must then fit by calling the refit method. If dostats is true
%   then the returned object is ready for doing stats otherwise you must
%   call the initstats method on the object to make it ready for doing
%   stats. If dofit is false then an unfitted object is returned and
%   dostats has no effect.
%
%   In most cases, you would set both dofit and dostats to true. If you
%   just want to access the estimated parameters thetaHat, betaHat, bHat,
%   sigmaHat, loglikHat, Psi then set dofit to true and dostats to false.
%   The idea is that computing stats may not be needed for certain
%   operations such as likelihood ratio tests.
%
%   Before calling any of the stats methods, both isFitToData and
%   isReadyForStats flags must be true.
%
%   slme = StandardLinearMixedModel(...,Name,Value,...) also supplies
%   optional name/value pairs:
%
%           Name                      Value
%           'Optimizer'               A string specifying the name of the 
%                                     optimizer. Supported values are
%                                     'quasinewton' and 'fminunc'.
%
%           'OptimizerOptions'        A structure containing the 
%                                     optimization options to be passed to 
%                                     the optimizer.
%
%           'InitializationMethod'    A string indicating how the initial
%                                     value of parameters should be chosen 
%                                     to initialize the optimizer.
%
%           'CheckHessian'            Either true or false. If true, we
%                                     perform positive definiteness checks
%                                     on (a) Hessian of the objective
%                                     function at convergence (b) Covariance 
%                                     of [thetaHat;log(sigmaHat)] and (c)
%                                     Covariance of [etaHat;log(sigmaHat)].
%                                     Default is false.            

            % (0) No arg constructor
            if ( nargin == 0 )
                return;
            end

            % (1) Ensure that dofit and dostats are scalar logicals.
            assert( isscalar(dofit) & islogical(dofit) );
            assert( isscalar(dostats) & islogical(dostats) );
            
            % (2) Validate inputs.
            [X,y,Z,Psi,FitMethod] = validateInputs(slme,X,y,Z,Psi,FitMethod);
            
            % (3) Set N, p and q.
            [N,p] = size(X); %#ok<*PROP>
            q = size(Z,2);            
            slme.N = N;
            slme.p = p;
            slme.q = q;
            
            % (4) Set X, y, Z, Psi and FitMethod.
            slme.X = X;
            slme.y = y;
            slme.Z = Z;
            slme.Psi = Psi;
            slme.FitMethod = FitMethod;            
            
            % (5) Parse optimization options.
                % (5a) Default optimization options.
                dfltOptimizer = 'quasinewton';
                dfltOptimizerOptions = struct([]);
                dfltInitializationMethod = 'default';
                dfltCheckHessian = false;
                
                % (5b) Optional parameter names and their default values.
                paramNames = {  'Optimizer',   'OptimizerOptions',   'InitializationMethod',   'CheckHessian'};
                paramDflts = {dfltOptimizer, dfltOptimizerOptions, dfltInitializationMethod, dfltCheckHessian};
           
                % (5c) Parse optional parameter name/value pairs.
                [optimizer,optimizeroptions,initializationmethod,checkhessian] ...
                    = internal.stats.parseArgs(paramNames,paramDflts,varargin{:});
            
                % 5(d) Validate optimization options.
                [optimizer,optimizeroptions,initializationmethod] ...
                    = validateOptimizationOptions(slme,optimizer,optimizeroptions,initializationmethod);
                
                % 5(e) Validate checkhessian.
                checkhessian = internal.stats.parseOnOff(checkhessian,'CheckHessian');
                
            % (6) Set optimization options in the object. Also store the
            % Hessian check option selected by the user.
            slme.Optimizer = optimizer;
            slme.OptimizerOptions = optimizeroptions;
            slme.InitializationMethod = initializationmethod;
            slme.CheckHessian = checkhessian;   
            
            % (7) Fit the standard LME model if asked.
            if (dofit == true)
                slme = refit(slme);
                % (8) Make the object ready for doing stats if required.
                if (dostats == true)
                    slme = initstats(slme);
                end
            end                        
            
        end % end of StandardLinearMixedModel.        
        
        function slme = refit(slme)
%refit - Fit and refit a Linear Mixed Effects (LME) model in standard form.
%   slme = refit(slme) fits a standard LME model previously initialized
%   with the StandardLinearMixedModel method. The public properties of slme
%   such as y, X, etc. can be changed if desired and the model can be refit
%   using the refit method. When changing X, y, Z, Psi or FitMethod, a call
%   to refit and initstats is necessary before calling any of the methods
%   that compute stats on the fitted model.                        

            % (1) Fit LME to get optimal theta and store it in the object.
            slme.thetaHat = solveForThetaHat(slme); 
            
            % (2) Find betaHat and bHat by solving the mixed model eqns. at 
            % slme.thetaHat. Then use betaHat, bHat and slme.thetaHat to 
            % compute sigmaHat and store the results in slme.            
            [slme.betaHat,slme.bHat,slme.sigmaHat] = ...
                solveMixedModelEquations(slme,slme.thetaHat);
            
            % (3) Set maximized log likelihood / restricted log likelihood.
            switch lower(slme.FitMethod)
                case 'ml'
                    slme.loglikHat = ...
                        BetaSigmaProfiledLogLikelihood(slme,slme.thetaHat);
                case 'reml'
                    slme.loglikHat = ...
                        SigmaProfiledRestrictedLogLikelihood(slme,slme.thetaHat);
                otherwise
                    % <entry key="BadFitMethod">''FitMethod'' must be either ''ML'' or ''REML''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadFitMethod'));
            end
            
            % (4) Update slme.Psi to contain optimized theta and sigma.
            slme.Psi = setUnconstrainedParameters(slme.Psi,slme.thetaHat);
            slme.Psi = setSigma(slme.Psi,slme.sigmaHat);                                                
            
            % (5) Mark the object as Fitted.
            slme.isFitToData = true;
            
        end % end of refit.        
        
        function slme = initstats(slme)
%initstats - Make a fitted StandardLinearMixedModel object ready for stats.
%   slme = initstats(slme) takes a fitted StandardLinearMixedModel object
%   slme and makes it ready for stats. slme must be a fitted object
%   returned by either StandardLinearMixedModel constructor or the refit
%   method.
            
            % (1) Ensure that slme has been fit to data.
            if (slme.isFitToData == false)
                % <entry key="MustRefitFirst">Call the method refit first and then initstats.</entry>
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:MustRefitFirst'));
            end

            % (2) Check the Hessian of objective function at convergence.
            if slme.CheckHessian == true
                checkObjectiveFunctionHessianAtThetaHat(slme);
            end
            
            % (3) Store the rank of X.
            slme.rankX = rank(slme.X);

            % (4) Store estimated covariance of betaHat in slme.
            slme.covbetaHat = ...
                covBetaHatAsFunctionOfThetaSigma(slme,slme.thetaHat,slme.sigmaHat);
            
            % (5) Store estimated covariance of [thetaHat;log(sigmaHat)] in
            % slme.
            slme.covthetaHatlogsigmaHat = covThetaHatLogSigmaHat(slme);

            % (6) Ensure that slme.covthetaHatlogsigmaHat is positive
            % definite.
            if slme.CheckHessian == true
                % <entry key="Message_NotSPDCovarianceUnconstrainedScale">The covariance matrix of covariance parameters on the unconstrained scale is not positive definite. This may indicate a model with more covariance parameters than those supported by data.</entry>
                msg1ID = 'stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_NotSPDCovarianceUnconstrainedScale';
                checkPositiveDefinite(slme.covthetaHatlogsigmaHat,msg1ID);
            end
            
            % (7) Store estimated covariance of [etaHat;log(sigmaHat)] in
            % slme.
            slme.covetaHatlogsigmaHat = covEtaHatLogSigmaHat(slme);
            
            % (8) Ensure that slme.covetaHatlogsigmaHat is positive
            % definite.
            if slme.CheckHessian == true
                % <entry key="Message_NotSPDCovarianceNaturalScale">The covariance matrix of covariance parameters on the Natural scale is not positive definite. This may indicate a model with more covariance parameters than those supported by data.</entry>
                msg2ID = 'stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_NotSPDCovarianceNaturalScale';
                checkPositiveDefinite(slme.covetaHatlogsigmaHat,msg2ID);
            end
            
            % TODO: This matrix may be very large. Decide whether to save
            % it in the object or compute it on the fly.
            % (9) Store estimated covariance of [betaHat-beta;bHat-b] in
            % slme.
            %slme.covbetaHatbHat = ...
            %    covBetaHatBHatAsFunctionOfThetaSigma(slme,slme.thetaHat,slme.sigmaHat);                        
            
            % (10) Mark the object as ready for stats.
            slme.isReadyForStats = true;            
            
        end % end of initstats.
        
        function slme = turnOffOptimizerDisplay(slme)
%turnOffOptimizerDisplay - Sets the display option to effective 'off'.            
%   slme = turnOffOptimizerDisplay(slme) takes a StandardLinearMixedModel
%   object slme, and depending on the value of slme.Optimizer, modifies the
%   slme.OptimizerOptions property such that subsequent calls to refit do
%   not display the convergence information. The modified object is
%   returned.

            switch lower(slme.Optimizer)
                case 'quasinewton'
                    slme.OptimizerOptions.Display = 'off';
                case 'fminunc'
                    slme.OptimizerOptions.Display = 'off';
                otherwise
                    % <entry key="BadOptimizerName">''Optimizer'' must be either ''quasinewton'' or ''fminunc''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadOptimizerName'));
            end

        end % turnOffOptimizerDisplay.
        
    end       

% Private methods related to computing various likelihoods. Checked OK.
    methods (Access=private)        
        
        function LogLik = LogLikelihood(slme,theta,sigma,beta)
%LogLikelihood - Compute the log likelihood of a StandardLinearMixedModel.
%   LogLik = LogLikelihood(slme,theta,sigma,beta) returns the log
%   likelihood of the StandardLinearMixedModel slme at the specified values
%   of theta, sigma and beta.

            % (1) Get X, y, Z, N and q from slme.
            X = slme.X;
            y = slme.y;
            Z = slme.Z;
            N = slme.N;
            q = slme.q;
           
            % (2) Get slme.Psi, set current theta and set sigma = 1.
            Psi = slme.Psi;
            Psi = setUnconstrainedParameters(Psi,theta);
            Psi = setSigma(Psi,1);

            % (3) Get lower triangular Cholesky factor of D matrix in Psi.
            % Lambda has size q by q where q = size(Z,2).
            Lambda = getLowerTriangularCholeskyFactor(Psi);

            % (4) Compute the U matrix. U is N by q.
            U = Z*Lambda;
            
            % TODO: Try to make steps 5 onward more efficient.
            % (5) Get R and S such that S*R'*R*S' = U'*U + eye(q) where R
            % and S are q by q. S is a permutation matrix: S*S' = eye(q).
            Iq = spdiags(ones(q,1),0,q,q);
            [R,status,S] = chol(U'*U + Iq);
 
            % (6) Make sure status is zero.
            if (status ~= 0)
                % <entry key="ErrorSparseCholesky">An error occured during sparse Cholesky factorization.</entry>
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:ErrorSparseCholesky'));
            end            
            
            % (7) Compute P1 (q by q).
            P1 = S*R';

            % (8) Compute Deltab for fixed beta.
            Deltab = P1' \ (P1 \ (U'*(y-X*beta)) );
            
            % (9) Compute b.
            b = Lambda * Deltab;
            
            % (10) Compute r2.
            r2 = sum(Deltab.^2) + sum((y - X*beta - Z*b).^2);
            
            % (11) Get log abs determinant of upper triangular matrix R.
            logAbsDetR = logAbsDetTriangular(R);

            % (12) Compute the log likelihood.
            sigma2 = sigma^2;
            LogLik = (-N/2)*log(2*pi*sigma2) - r2/(2*sigma2) - logAbsDetR;            

        end % end of LogLikelihood.
        
        function PLogLik = BetaSigmaProfiledLogLikelihood(slme,theta)
%BetaSigmaProfiledLogLikelihood - Compute the profiled log likelihood with beta and sigma profiled out.
%   PLogLik = BetaSigmaProfiledLogLikelihood(slme,theta) returns the 
%   profiled log likelihood of a StandardLinearMixedModel slme at the 
%   specified value of theta. Both beta and sigma have been profiled out.           

            % (1) Using the given value of theta, solve mixed model 
            % equations to get r2 and R.
            [~,~,~,r2,R,~] = solveMixedModelEquations(slme,theta);
            
            % (2) Get log abs determinant of upper triangular matrix R.
            logAbsDetR = logAbsDetTriangular(R);

            % (3) Compute profiled log likelihood.
            N = slme.N;
            PLogLik = (-N/2)*( 1 + log( 2*pi*r2/N ) ) - logAbsDetR;
            
        end % end of BetaSigmaProfiledLogLikelihood.
        
        function PLogLik = BetaProfiledLogLikelihood(slme,theta,sigma)
%BetaProfiledLogLikelihood - Compute the profiled log likelihood with beta profiled out.
%   PLogLik = BetaProfiledLogLikelihood(slme,theta,sigma) returns the
%   profiled log likelihood of a StandardLinearMixedModel slme at the
%   specified values of theta and sigma with beta profiled out.

            % (1) Using the given value of theta, solve mixed model 
            % equations to get r2 and R.
            [~,~,~,r2,R,~] = solveMixedModelEquations(slme,theta);
            
            % (2) Get log abs determinant of upper triangular matrix R.
            logAbsDetR = logAbsDetTriangular(R);

            % (3) Compute profiled log likelihood (sigma not profiled out).
            N = slme.N;
            sigma2 = sigma^2;
            PLogLik = (-N/2)*log(2*pi*sigma2) - r2/(2*sigma2) - logAbsDetR;
            
        end % end of BetaProfiledLogLikelihood.                
        
        function PRLogLik = SigmaProfiledRestrictedLogLikelihood(slme,theta)
%SigmaProfiledRestrictedLogLikelihood - Compute the profiled restricted log likelihood.
%   PRLogLik = SigmaProfiledRestrictedLogLikelihood(slme,theta) returns the
%   profiled restricted log likelihood of a StandardLinearMixedModel slme
%   at the specified value of theta with sigma profiled out.

            % (1) Using the given value of theta, solve mixed model
            % equations to get r2, R and R1.
            [~,~,~,r2,R,R1] = solveMixedModelEquations(slme,theta);
            
            % (2) Get log abs determinant of upper triangular R and lower
            % triangular R1.
             logAbsDetR = logAbsDetTriangular(R);
            logAbsDetR1 = logAbsDetTriangular(R1);

            % (3) Compute profiled restricted log likelihood.
            N = slme.N;
            p = slme.p;
            PRLogLik = (-(N-p)/2) * ( 1 + log( 2*pi*r2/(N-p) ) ) ...
                - logAbsDetR - logAbsDetR1;            

        end % end of SigmaProfiledRestrictedLogLikelihood.        
        
        function RLogLik = RestrictedLogLikelihood(slme,theta,sigma)
%RestrictedLogLikelihood - Compute the restricted log likelihood. 
%   RLogLik = RestrictedLogLikelihood(slme,theta,sigma) returns the
%   restricted log likelihood of a StandardLinearMixedModel at the
%   specified values of theta and sigma.

            % (1) Using the given value of theta, solve mixed model
            % equations to get r2, R and R1.
            [~,~,~,r2,R,R1] = solveMixedModelEquations(slme,theta);
            
            % (2) Get log abs determinant of upper triangular R and lower
            % triangular R1.
             logAbsDetR = logAbsDetTriangular(R);
            logAbsDetR1 = logAbsDetTriangular(R1);

            % (3) Compute restricted log likelihood.
            N = slme.N;
            p = slme.p;
            sigma2 = sigma^2;
            RLogLik = (-(N-p)/2)*log(2*pi*sigma2) - r2/(2*sigma2) ...
                - logAbsDetR - logAbsDetR1;
            
        end % end of RestrictedLogLikelihood.                                                      
        
    end       
        
% Covariance of betaHat and [betaHat - beta;bHat - b] as functions of theta and sigma. Checked OK.    
    methods (Access=private)                                                                                                             
                          
        function covbetaHat = covBetaHatAsFunctionOfThetaSigma(slme,theta,sigma)
%covBetaHatAsFunctionOfThetaSigma - Covariance matrix of betaHat as function of theta and sigma.
%   covbetaHat = covBetaHatAsFunctionOfThetaSigma(slme,theta,sigma) returns
%   the covariance matrix of betaHat as a function of theta and sigma.
            
            % (1) Given theta, get R1.
            [~,~,~,~,~,R1] = solveMixedModelEquations(slme,theta);
            
            % (2) Return covbetaHat = sigma^2 * inv(R1*R1') 
            %                       = sigma^2 * inv(R1)'*inv(R1)
            sigma2 = sigma^2;
            invR1 = R1 \ eye(size(R1));
            covbetaHat = sigma2*(invR1'*invR1);
            
        end % end of covBetaHatAsFunctionOfThetaSigma.
        
        function f = makecovcTBetaHatAsFunctionOfThetaLogSigma(slme,c)   
%   f = makecovcTBetaHatAsFunctionOfThetaLogSigma(slme,c) takes a p by 1
%   vector c and gets the covariance of c'*betaHat as a function of
%   [theta;log(sigma)].

                f = @f4;
                function ret = f4(x)
                    theta = x(1:end-1);
                    logsigma = x(end);
                    sigma = exp(logsigma);                    
                    ret = c'*covBetaHatAsFunctionOfThetaSigma(slme,theta,sigma)*c;                    
                end                
                
        end % makecovcTBetaHatAsFunctionOfThetaLogSigma.
                       
        function covbetaHatbHat = covBetaHatBHatAsFunctionOfThetaSigma(slme,theta,sigma)
%   covbetaHatbHat = covBetaHatBHatAsFunctionOfThetaSigma(slme,theta,sigma) 
%   returns the covariance of [betaHat - beta;bHat - b] as a function of
%   theta and sigma.

            % (1) Get X, Z, p and q
            X = slme.X;
            Z = slme.Z;
            p = slme.p;
            q = slme.q;
            
            % (2) Get Psi from slme and set its unconstrained parameters.
            Psi = slme.Psi;
            Psi = Psi.setUnconstrainedParameters(theta);
            
            % (3) Get lower triangular Cholesky factor from Psi.
            Lambda = getLowerTriangularCholeskyFactor(Psi);
            
            % (4) Get U.
            U = Z*Lambda;
            
            % (5) Form the matrix [X'*X X'*U;U'*X U'*U + I].
            M = zeros(p+q);
            M(1:p,1:p) = X'*X;
            M(1:p,p+1:end) = X'*U;
            M(p+1:end,1:p) = M(1:p,p+1:end)';
            M(p+1:end,p+1:end) = U'*U + eye(q);
            
            % (6) Get Cholesky factor C such that C*C' = M.
            try 
                C = chol(M,'lower');
            catch ME %#ok<NASGU>
                C = classreg.regr.lmeutils.covmats.CovarianceMatrix.singularLowerChol(M);
            end
            
            % (7) Get G = [I_p 0;0 Lambda^T].
            G = zeros(p+q);
            G(1:p,1:p) = eye(p);
            G(p+1:end,p+1:end) = Lambda';
                        
            % (8) Get T = inv(C) * G.
            T = C \ G;
            
            % (9) Get sigma^2 * (T'*T) as the required covariance.
            covbetaHatbHat = (sigma^2)*(T'*T);
                    
        end % end of covBetaHatBHatAsFunctionOfThetaSigma.        
        
        function f = makecovcTBetaHatBHatAsFunctionOfThetaLogSigma(slme,c)
%   f = makecovcTBetaHatBHatAsFunctionOfThetaLogSigma(slme,c) takes a (p+q)
%   by 1 vector c and gets the covariance of c'*[betaHat;bHat] as a
%   function of [theta;log(sigma)].

                f = @f5;
                function ret = f5(x)
                    theta = x(1:end-1);
                    logsigma = x(end);
                    sigma = exp(logsigma);
                    ret = c'*covBetaHatBHatAsFunctionOfThetaSigma(slme,theta,sigma)*c;
                end
            
        end % end of makecovcTBetaHatBHatAsFunctionOfThetaLogSigma.        
    
    end
        
% Private method to get covariance of [thetaHat;log(sigmaHat)]. Checked OK.   
    methods (Access=private)
        
        function C = covThetaHatLogSigmaHat(slme)
%covThetaHatLogSigmaHat - Estimated covariance of [thetaHat;log(sigmaHat)]. 
%   C = covThetaHatLogSigmaHat(slme) returns the estimated covariance
%   matrix of [thetaHat;log(sigmaHat)].

            % (1) Define x = [slme.thetaHat;log(slme.sigmaHat)].
            x = [slme.thetaHat;log(slme.sigmaHat)];

            % (2) Get beta profiled/restricted log likelihood as required.
            switch lower(slme.FitMethod)                                            
               case 'ml'
                   % (2) Get Hessian of negative profiled log likelihood as
                   % a function of [theta;log(sigma)] with beta profiled
                   % out at x = [slme.thetaHat;log(slme.sigmaHat)].
                   fun = makeNegBetaProfiledLogLikelihoodAsFunctionOfThetaLogSigma(slme);                  
               case 'reml'               
                   % (2) Get Hessian of negative restricted log likelihood
                   % as a function of [theta;log(sigma)] at x =
                   % [slme.thetaHat;log(slme.sigmaHat)].                   
                   fun = makeNegRestrictedLogLikelihoodAsFunctionOfThetaLogSigma(slme);                   
            end
            
            % (3) Get Hessian of fun at x. 
            wantRegularized = false;
            H = getHessian(fun,x,wantRegularized);  
            
            % (4) Invert H to get the covariance matrix. H may be singular. 
            % Turn warning MATLAB:singularMatrix off, do the inversion and 
            % restore the warning state.
            warnState = warning('query','all');
            warning('off','MATLAB:singularMatrix');
            warning('off','MATLAB:illConditionedMatrix');
            warning('off','MATLAB:nearlySingularMatrix');
            cleanupObj = onCleanup(@() warning(warnState));
            
            C = H \ eye(size(H));               
            %C = pinv(H);            
                       
        end % covThetaHatLogSigmaHat.        
        
    end
   
% Private method to get covariance of [etaHat,log(sigmaHat)]. Checked OK.
    methods (Access=private)
        
        function C = covEtaHatLogSigmaHat(slme)
%covEtaHatLogSigmaHat - Estimated covariance of [EtaHat;log(sigmaHat)]. 
%   C = covEtaHatLogSigmaHat(slme) returns the estimated covariance
%   matrix of [etaHat;log(sigmaHat)] where etaHat is the estimated Natural
%   parameter vector.            
            
            % (1) Get etaHat and sigmaHat.
            etaHat = getNaturalParameters(slme.Psi);
            sigmaHat = slme.sigmaHat;

            % (2) Define x = [etaHat;log(sigmaHat)].
            x = [etaHat;log(sigmaHat)];

            % (3) Get beta profiled/restricted log likelihood as required.
            switch lower(slme.FitMethod)                                            
               case 'ml'
                   % (3) Get Hessian of negative profiled log likelihood as
                   % a function of [eta;log(sigma)] with beta profiled
                   % out at x.
                   fun = makeNegBetaProfiledLogLikelihoodAsFunctionOfEtaLogSigma(slme);                  
               case 'reml'               
                   % (3) Get Hessian of negative restricted log likelihood
                   % as a function of [eta;log(sigma)] at x.                   
                   fun = makeNegRestrictedLogLikelihoodAsFunctionOfEtaLogSigma(slme);                   
            end
            
            % (4) Get Hessian of fun at x.
            wantRegularized = false;
            try
                H = getHessian(fun,x,wantRegularized);
            catch ME %#ok<NASGU>
                H = NaN(length(x));
            end
            
            % (5) Invert H to get the covariance matrix. H may be singular. 
            % Turn warning MATLAB:singularMatrix off, do the inversion and 
            % restore the warning state.
            warnState = warning('query','all');
            warning('off','MATLAB:singularMatrix');
            warning('off','MATLAB:illConditionedMatrix');
            warning('off','MATLAB:nearlySingularMatrix');
            cleanupObj = onCleanup(@() warning(warnState));
            
            try
                C = covarianceOnNaturalScale(slme,H);
                %C = H \ eye(size(H));
            catch ME %#ok<NASGU>
                C = H \ eye(size(H));
            end
            %C = pinv(H);
            
        end % end of covEtaHatLogSigmaHat.
        
        
        function C = covarianceOnNaturalScale(slme,H) %#ok<INUSL>
%covarianceOnNaturalScale - Approximate the covariance of natural parameters.
%   C = covarianceOnNaturalScale(H) takes a symmetric matrix H where H is
%   either the Hessian of the negative profiled log likelihood (beta
%   profiled out) or the Hessian of the negative restricted log likelihood
%   as a function of [eta;log(sigma)] evaluated at [etaHat;log(sigmaHat)].
%   The covariance C is given by inv(H) except that here we account for
%   cases when H is not positive definite.

            % (1) Tentatively, set C as the pseudo inverse of H.
            C = pinv(H);

            % (2) Which elements of the Natural parameter vector are
            % estimable considering H as equivalent to the Normal matrix
            % X'*X in linear regression.
            sizeH = size(H,1);
            F = internal.stats.isEstimable(eye(sizeH),'NormalMatrix',H);

            % (3) If C(i,i) is < 0, F(i) must be set to false since the ith 
            % element of Natural parameter vector cannot have a negative
            % variance.
            diagC = diag(C);
            F(diagC < 0) = false;
            
            % (4) Set elements of non-estimable rows and columns as NaN.
            C(~F,:) = NaN;
            C(:,~F) = NaN;            

        end % end of covarianceOnNaturalScale.
        
    end
    
% Private method to get CIs on the Natural parameter vector. Checked OK.
    methods (Access=private)
       
        function CI = etaLogSigmaCI(slme,alpha)
%etaLogSigmaCI - Confidence intervals on the Natural parameter vector.
%   CI = etaLogSigmaCI(slme,alpha) takes a StandardLinearMixedModel object
%   slme and a confidence level between 0 and 1 and computes the
%   (1-alpha)*100 CIs for [eta;log(sigma)]. CI(:,1) contains the lower
%   limits of the CIs and CI(:,2) contains the upper limits of the CIs. The
%   last row of CI contains the lower and upper limit of the CI for
%   log(sigma).

            % (1) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );
            
            % (2) Form x = [etaHat;log(sigmaHat)].
            etaHat = getNaturalParameters(slme.Psi);
            sigmaHat = slme.sigmaHat;
            x = [etaHat;log(sigmaHat)];
            
            % (3) Get covariance of [etaHat;log(sigmaHat)].
            C = slme.covetaHatlogsigmaHat;
            
            % (4) (1-alpha) confidence interval on elements of 
            % [eta;log(sigma)]. CI will have length(etaHat) + 1 rows and 2 
            % columns.
            delta = norminv(1-alpha/2)*sqrt(diag(C));
            CI = [x - delta,x + delta];            
            
        end % end of etaLogSigmaCI.
        
    end
    
% Public method to get CIs on the Canonical parameter vector. Checked OK.
    methods (Access=public)
       
        function CI = hetaSigmaCI(slme,alpha)
%hetaSigmaCI - Confidence intervals on the Canonical parameters.
%   CI = hetaSigmaCI(slme,alpha) takes a StandardLinearMixedModel object
%   slme and a confidence level between 0 and 1 and computes the
%   (1-alpha)*100 CIs for elements of the vector [heta;sigma] where heta is
%   the Canonical parameter vector. CI(:,1) contains the lower limits of
%   the CIs and CI(:,2) contains the upper limits of the CIs. The last row
%   of CI contains the lower and upper limit of the CI for sigma.
        
            % (1) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );

            % (2) Get CIs on [eta;log(sigma)].
            CI0 = etaLogSigmaCI(slme,alpha);
            
            % (3) Get the Psi object from slme.
            Psi = slme.Psi;
            
            % (3) CI0(:,1) contains lower bounds for a (1-alpha) CI on
            % [eta;log(sigma)]. Transform CI0(:,1) to a lower bound on
            % heta and sigma.
            eta_lb = CI0(1:end-1,1);
            logsigma_lb = CI0(end,1);
            sigma_lb = exp(logsigma_lb);
            Psi = setNaturalParameters(Psi,eta_lb);
            Psi = setSigma(Psi,sigma_lb);
            heta_lb = getCanonicalParameters(Psi);
            
            % (4) Transform CI0(:,2) to an upper bound on heta and sigma.
            eta_ub = CI0(1:end-1,2);
            logsigma_ub = CI0(end,2);
            sigma_ub = exp(logsigma_ub);
            Psi = setNaturalParameters(Psi,eta_ub);
            Psi = setSigma(Psi,sigma_ub);
            heta_ub = getCanonicalParameters(Psi);
            
            % (5) Assemble CI.
            CI(:,1) = [heta_lb;sigma_lb];
            CI(:,2) = [heta_ub;sigma_ub];                        
            
        end % end of hetaSigmaCI.
        
    end
    
% Private method to get estimated covariance of [betaHat - beta;bHat - b]. Checked OK.   
    methods (Access=private)                                    
        
        function C = covBetaHatBHat(slme)
%covBetaHatBHat - Estimated covariance matrix of [betaHat-beta; bHat-b].          
%   C = covBetaHatBHat(slme) returns the estimated covariance matrix of
%   [betaHat - beta; bHat - b].
            
            % Calculate the covariance if not already stored in object.
            if isempty(slme.covbetaHatbHat)
                C = covBetaHatBHatAsFunctionOfThetaSigma(slme,...
                    slme.thetaHat,slme.sigmaHat);
            else
                C = slme.covbetaHatbHat;
            end            
            
        end % end of covBetaHatBHat.
                        
    end

% Public method to store/unstore the covariance of [betaHat - beta;bHat - b] in object.
    methods (Access=public)
        
        function slme = storeCovBetaHatBHat(slme)
%storeCovBetaHatBHat - Store the covariance of [betaHat - beta;bHat - b] in object.
%   slme = storeCovBetaHatBHat(slme) takes an object slme of type
%   StandardLinearMixedModel and stores the covariance of 
%   [betaHat - beta;bHat - b] in the object and returns the modified
%   object.

            slme.covbetaHatbHat = covBetaHatBHat(slme);

        end % end of storeCovBetaHatBHat.
        
        function slme = unstoreCovBetaHatBHat(slme)
%unstoreCovBetaHatBHat - Remove the stored covariance of [betaHat - beta;bHat - b] from object.
%   slme = unstoreCovBetaHatBHat(slme) takes an object slme of type
%   StandardLinearMixedModel and removes the stored covariance of 
%   [betaHat - beta;bHat - b] from the object and returns the modified
%   object.

            slme.covbetaHatbHat = [];

        end % end of storeCovBetaHatBHat.
        
    end
    
% Private methods to get Satterthwaite DF for T and F tests on beta vector. Checked OK.
    methods (Access=public)
                
        function df = dfBetaTTest(slme,c)
%dfBetaTTest - Satterthwaite degrees of freedom for testing c'*beta = e.
%   df = dfBetaTTest(slme,c) returns the Satterthwaite denominator degrees
%   of freedom for the test: c'*beta = e. beta is a p by 1 vector and so c 
%   must be a p by 1 vector as well. The calculation is insensitive to the 
%   value of e.
    
            % (1) Ensure that c has the right size.
            if size(c,1) == 1
                c = c';
            end
            assert( all(size(c) == [slme.p,1]) );
            
            % (2) If V denotes the covariance of betaHat at the values 
            % (theta,sigma) then make the function c'*V(theta,sigma)*c as a
            % function of [theta;log(sigma)].
            ctVcfun = makecovcTBetaHatAsFunctionOfThetaLogSigma(slme,c);            
            
            % (3) Let x = [slme.thetaHat;log(slme.sigmaHat)].
            x = [slme.thetaHat;log(slme.sigmaHat)];
            
            % (4) gHat is the gradient of ctVcfun evaluated at x.
            gHat = getGradient(ctVcfun,x);
            
            % (5) Get CHat the covariance matrix of [thetaHat;log(sigmaHat)]
            % at x. This has been already computed by refit.
            CHat = slme.covthetaHatlogsigmaHat;
            
            % (6) Compute df.
            df = 2*(ctVcfun(x))^2/(gHat'*CHat*gHat);            
            
            % (7) Ensure that df is never < 0.
            df = max(0,df);
            
        end % end dfBetaTTest.                        
        
        function df = dfBetaFTest(slme,L)
%dfBetaFTest - Satterthwaite degrees of freedom for testing L*beta = e.           
%   df = dfBetaFTest(slme,L) returns the Satterthwaite denominator degrees 
%   of freedom for the test L*beta = e. beta is the p by 1 vector and L 
%   must be a r by p matrix with r <= p and rank(L) = r. The calculation is
%   insensitive to the value of e.
            
           % (1) Make sure L is sensible.
           r = size(L,1);
           assert( size(L,2) == slme.p & r <= slme.p );
           assert( rank(L) == r );

           % (2) Get the estimated covariance of betaHat.
           V = slme.covbetaHat;
           
           % (3) Get eigendecomposition of L*V*L' = Veig*Lambdaeig*Veig'
           [Veig,Lambdaeig] = eig(L*V*L');
           Lambdaeig = diag(Lambdaeig);
           
           % (4) Get U matrix.
           U = Veig';
           
           % (5) Get B such that B(i,:) = b_i^T.
           B = U*L;
           
           % (6) Get [thetaHat;log(sigmaHat)].
           x = [slme.thetaHat;log(slme.sigmaHat)];
           
           % (7) Get covariance of [thetaHat;log(sigmaHat)].
           C = slme.covthetaHatlogsigmaHat;
           
           % (8) Initialize nu and loop.
           nu = zeros(r,1);           
           for i = 1:r
                % (9) Get covariance of b_i'*betaHat as a function of
                % [theta;log(sigma)].
                bi = B(i,:)';
                fi = makecovcTBetaHatAsFunctionOfThetaLogSigma(slme,bi);
                
                % (10) Get gradient of fi at x = [thetaHat;log(sigmaHat)].
                gi = getGradient(fi,x);
                
                % (11) Get nu(i)
                nu(i) = 2*(Lambdaeig(i)^2)/(gi'*C*gi);                
           end
           
           % (12) Compute G.
           nu = nu( (nu > 2) );
           if isempty(nu)
                G = 0;
           else
                G = sum( (nu./(nu-2)) );
           end
           %G = sum( (nu./(nu-2)) .* (nu > 2) );
           
           % (13) Compute df.
           if ( G > r )
                df = 2*G/(G-r);            
           else
                df = 0;
           end
           
        end % end of dfBetaFTest.
        
    end

% Private methods to get Satterthwaite DF for T and F tests on [beta;b] vector. Checked OK.
    methods (Access=public)
        
        function df = dfBetaBTTest(slme,c)
%dfBetaBTTest - Satterthwaite degrees of freedom for testing c'*[beta;b] = e.
%   df = dfBetaBTTest(slme,c) returns the Satterthwaite denominator degrees
%   of freedom for the test: c'*[beta;b] = e. beta is a p by 1 vector and b
%   is a q by 1 vector and c must be a (p+q) by 1 vector. The calculation
%   is insensitive to the value of e.
            
            % (1) Ensure that c has the right size.
            if size(c,1) == 1
                c = c';
            end
            assert( all(size(c) == [slme.p+slme.q,1]) );
            
            % (2) If V denotes the covariance of [betaHat-beta;bHat-b] at
            % the values (theta,sigma) then make the function
            % c'*V(theta,sigma)*c as a function of [theta;log(sigma)].
            ctVcfun = makecovcTBetaHatBHatAsFunctionOfThetaLogSigma(slme,c);
            
            % (3) Let x = [slme.thetaHat;log(slme.sigmaHat)].
            x = [slme.thetaHat;log(slme.sigmaHat)];
            
            % (4) gHat is the gradient of ctVcfun evaluated at x.
            gHat = getGradient(ctVcfun,x);
            
            % (5) Get CHat the covariance matrix of [thetaHat;log(sigmaHat)]
            % at x. This has been already computed by refit.
            CHat = slme.covthetaHatlogsigmaHat;
            
            % (6) Compute df.
            df = 2*(ctVcfun(x))^2/(gHat'*CHat*gHat);
            
            % (7) Ensure that df is never < 0.
            df = max(0,df);
            
        end %end of dfBetaBTTest.
        
        function df = dfBetaBFTest(slme,L)
%dfBetaBFTest - Satterthwaite degrees of freedom for testing L*[beta;b] = e.           
%   df = dfBetaBFTest(slme,L) returns the Satterthwaite denominator degrees 
%   of freedom for the test L*[beta;b] = e. beta is the p by 1 vector, b is
%   a q by 1 vector and L is a r by (p+q) matrix with r <= (p+q) and with 
%   rank(L) = r. The calculation is insensitive to the value of e.            
            
           % (1) Make sure L is sensible.
           r = size(L,1);
           assert( size(L,2) == (slme.p+slme.q) & r <= (slme.p+slme.q) );
           assert( rank(L) == r );

           % (2) Get the estimated covariance of [betaHat-beta;bHat - b].
           V = covBetaHatBHatAsFunctionOfThetaSigma(slme,slme.thetaHat,slme.sigmaHat);
           
           % (3) Get eigendecomposition of L*V*L' = Veig*Lambdaeig*Veig'
           [Veig,Lambdaeig] = eig(L*V*L');
           Lambdaeig = diag(Lambdaeig);
           
           % (4) Get U matrix.
           U = Veig';
           
           % (5) Get B such that B(i,:) = b_i^T.
           B = U*L;
           
           % (6) Get [thetaHat;log(sigmaHat)].
           x = [slme.thetaHat;log(slme.sigmaHat)];
           
           % (7) Get covariance of [thetaHat;log(sigmaHat)].
           C = slme.covthetaHatlogsigmaHat;
           
           % (8) Initialize nu and loop.
           nu = zeros(r,1);           
           for i = 1:r
                % (9) Get covariance of b_i'*betaHat as a function of
                % [theta;log(sigma)].
                bi = B(i,:)';
                fi = makecovcTBetaHatBHatAsFunctionOfThetaLogSigma(slme,bi);
                % (10) Get gradient of fi at x = [thetaHat;log(sigmaHat)].
                gi = getGradient(fi,x);
                
                % (11) Get nu(i)
                nu(i) = 2*(Lambdaeig(i)^2)/(gi'*C*gi);                
           end
           
           % (12) Compute G.
           G = sum( (nu./(nu-2)) .* (nu > 2) );
           
           % (13) Compute df.
           if ( G > r )
                df = 2*G/(G-r);            
           else
                df = 0;
           end

        end % end dfBetaBFTest.        
        
    end
    
% Public method for testing c'*beta = e. (T-test). Checked OK.
    methods (Access=public)
        
        function [P,T,DF] = betaTTest(slme,c,e,dfmethod)
%betaTTest - T-test for testing c'*beta = e.            
%   [P,T,DF] = betaTTest(slme,c,e,dfmethod) takes an object of type
%   StandardLinearMixedModel slme, a p by 1 vector c, a scalar e and a
%   string dfmethod which is one of 'none', 'residual' or 'satterthwaite'
%   and performs the T-test for testing H0: c'*beta = e. P is the two sided
%   p-value for the test, T is the test statistic and DF is the denominator
%   degrees of freedom used in the T-test.
    
            % TODO: Account for an all zero c.
            % (1) Ensure that c is sensible.
            if size(c,1) == 1
                c = c';
            end
            assert( all(size(c) == [slme.p,1]) );
            
            % (2) Ensure that e is sensible.            
            assert( all(size(e) == [1,1]) );
            
            % (3) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );
            
            % (4) Compute the df based on dfmethod.
            switch dfmethod
                case 'none'
                    DF = Inf;
                case 'residual'
                    DF = slme.N - slme.rankX;
                case 'satterthwaite'
                    DF = dfBetaTTest(slme,c);
                otherwise
                    % <entry key="BadDFMethod">Degrees of freedom method must be ''None'', ''Residual'' or ''Satterthwaite''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadDFMethod'));
            end
            
            % (5) Compute the test statistic T.
            T = (c'*slme.betaHat - e)/sqrt(c'*slme.covbetaHat*c);
            
            % (6) Compute the two sided p-value: P = 2*Pr(t_df > abs(T)).
            % P = 2*(1 - tcdf(abs(T),DF));
            P = 2*(tcdf(abs(T),DF,'upper'));
            
        end % end of betaTTest.
        
    end
    
% Public method for testing t'*beta + s'*b = e (T-test). Checked OK.
    methods (Access=public)
       
        function [P,T,DF] = betaBTTest(slme,t,s,e,dfmethod)
%betaBTTest - T-test for testing t'*beta + s'*b = e.            
%   [P,T,DF] = betaBTTest(slme,t,s,e,dfmethod) takes an object of type
%   StandardLinearMixedModel slme, a p by 1 vector t, a q by 1 vector s, a
%   scalar e and a string dfmethod which is one of 'none', 'residual' or
%   'satterthwaite' and performs the T-test for testing H0: t'*beta + s'*b
%   = e. P is the two sided p-value for the test, T is the test statistic
%   and DF is the denominator degrees of freedom used in the T-test.
            
            % (1) Ensure that t is sensible.
            if size(t,1) == 1
                t = t';
            end
            assert( all(size(t) == [slme.p,1]) );
            
            % (2) Ensure that s is sensible.
            if size(s,1) == 1
                s = s';
            end
            assert( all(size(s) == [slme.q,1]) );
            
            % (3) Ensure that e is sensible.
            assert( all(size(e) == [1,1]) );
            
            % (4) Get the joint contast vector c.
            c = [t;s];                        
                        
            % (5) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );

            % (6) Compute the df based on dfmethod.
            switch dfmethod
                case 'none'
                    DF = Inf;
                case 'residual'
                    DF = slme.N - slme.rankX;
                case 'satterthwaite'
                    DF = dfBetaBTTest(slme,c);
                otherwise
                    % <entry key="BadDFMethod">Degrees of freedom method must be ''None'', ''Residual'' or ''Satterthwaite''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadDFMethod'));
            end
            
            % (7) Get the joint vector [betaHat;bHat].
            betaHatbHat = [slme.betaHat;slme.bHat];
            
            % (8) Get the covariance of [betaHat-beta;bHat-b].
            % TO DO: Avoid computing V from scratch. Figure out how to
            % store this potentially large matrix in the object.
            V = covBetaHatBHat(slme);
            
            % (9) Compute the test statistic T.
            T = (c'*betaHatbHat - e)/sqrt(c'*V*c);
            
            % (10) Compute the two sided p-value: P = 2*Pr(t_df > abs(T)).
            % P = 2*(1 - tcdf(abs(T),DF));   
            P = 2*(tcdf(abs(T),DF,'upper'));

        end % end of betaBTTest.
        
    end
        
% Public method for testing L*beta = e. (F-test). Checked OK.
    methods (Access=public)
        
        function [P,T,DF1,DF2] = betaFTest(slme,L,e,dfmethod)
%betaFTest - F-test for testing L*beta = e.            
%   [P,T,DF1,DF2] = betaFTest(slme,L,e,dfmethod) takes an object of type
%   StandardLinearMixedModel slme, a r by p matrix L of rank r, a r by 1
%   vector e and a string dfmethod which is one of 'none', 'residual' or
%   'satterthwaite' and performs the F-test for testing H0: L*beta = e. P
%   is the p-value for the test, T is the test statistic, DF1 is the
%   numerator degrees of freedom and DF2 is the denominator degrees of
%   freedom used in the F-test.

            % (1) Ensure that L is sensible.
            assert( size(L,2) == slme.p );

            % (2) Ensure that e is sensible.
            if size(e,1) == 1
                e = e';
            end
            assert( all(size(e) == [size(L,1),1]) );
            
            % (3) Handle the empty L or e case.
            if isempty(L)
                  P = NaN;
                  T = NaN;
                DF1 = NaN;
                DF2 = NaN;
                return;
            end
            
            % (4) Get effective L and e.
            %
            % L may not be of full row rank and the hypothesis L*beta = e 
            % may not be consistent. Get the effective L and e for the test
            % and ensure that original L and e are consistent.            
            %       L = r by slme.p matrix
            %       e = r by 1 vector.
            % Desired test: 
            %       L*beta = e.            
            [ok,~,L,e]=fullrankH(L,e);                        
            if ~ok
                % <entry key="BadHypothesisMatrix">The hypothesis matrix and test value are not consistent.</entry>
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadHypothesisMatrix'));
            end
            r = size(L,1);
            
            % (5) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );
            
            % (6) Compute the denominator df based on dfmethod.
            switch dfmethod
                case 'none'
                    DF2 = Inf;
                case 'residual'
                    DF2 = slme.N - slme.rankX;
                case 'satterthwaite'
                    DF2 = dfBetaFTest(slme,L);
                otherwise
                    % <entry key="BadDFMethod">Degrees of freedom method must be ''None'', ''Residual'' or ''Satterthwaite''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadDFMethod'));
            end
            
            % (7) Compute the test statistic T.
            delta = (L*slme.betaHat - e);
            T = delta'*( (L*slme.covbetaHat*L') \ delta )/r;
            
            % (8) Set numerator degrees of freedom as r.
            DF1 = r;
            
            % (9) Compute p-value for the F-test.
            % P = 1 - fcdf(T,DF1,DF2);
            P = fcdf(T,DF1,DF2,'upper');
            
        end % end of betaFTest.
        
    end

% Public method for testing H*beta + K*b = e. (F-test). Checked OK.
    methods (Access=public)
        
        function [P,T,DF1,DF2] = betaBFTest(slme,H,K,e,dfmethod)
%betaBFTest - F-test for testing H*beta + K*b = e.            
%   [P,T,DF1,DF2] = betaBFTest(slme,H,K,e,dfmethod) takes an object of type
%   StandardLinearMixedModel slme, a r by p matrix H, a r by q matrix K, a
%   r by 1 vector e and a string dfmethod which is one of 'none',
%   'residual' or 'satterthwaite' and performs the F-test for testing H0:
%   H*beta + K*b = e. P is the p-value for the test, T is the test
%   statistic, DF1 is the numerator degrees of freedom and DF2 is the
%   denominator degrees of freedom used in the F-test. The rank of matrix
%   [H,K] must be r <= (p + q). 

            % (1) Ensure that H is sensible.
            assert( size(H,2) == slme.p );
                        
            % (2) Ensure that K is sensible.
            assert( size(K,1) == size(H,1) );
            assert( size(K,2) == slme.q );
            
            % (3) Form contrast matrix L for [beta;b].
            L = [H,K];                        
            
            % (4) Ensure that e is sensible.
            if size(e,1) == 1
                e = e';
            end
            assert( all(size(e) == [size(H,1),1]) );
            
            % (5) Get effective L and e.
            %
            % L may not be of full row rank and the hypothesis L*[beta;b] =
            % e may not be consistent. Get the effective L and e for the
            % test and ensure that original L and e are consistent.
            %       L = r by [slme.p + slme.q] matrix
            %       e = r by 1 vector.
            % Desired test: 
            %       L*[beta;b] = e.            
            [ok,~,L,e]=fullrankH(L,e);                        
            if ~ok
                % <entry key="BadHypothesisMatrix">The hypothesis matrix and test value are not consistent.</entry>               
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadHypothesisMatrix'));
            end
            r = size(L,1);            
            
            % (6) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );
            
            % (7) Compute the denominator df based on dfmethod.
            switch dfmethod
                case 'none'
                    DF2 = Inf;
                case 'residual'
                    DF2 = slme.N - slme.rankX;
                case 'satterthwaite'
                    DF2 = dfBetaBFTest(slme,L);
                otherwise
                    % <entry key="BadDFMethod">Degrees of freedom method must be ''None'', ''Residual'' or ''Satterthwaite''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadDFMethod'));
            end
            
            % (8) Get the joint vector [betaHat;bHat].
            betaHatbHat = [slme.betaHat;slme.bHat];
            
            % (9) Get the covariance of [betaHat-beta;bHat-b].
            V = covBetaHatBHat(slme);
                                    
            % (10) Compute the test statistic T.
            delta = (L*betaHatbHat - e);
            T = delta'*( (L*V*L') \ delta )/r;
            
            % (11) Set numerator degrees of freedom as r.
            DF1 = r;
            
            % (12) Compute p-value for the F-test.
            % P = 1 - fcdf(T,DF1,DF2);
            P = fcdf(T,DF1,DF2,'upper');
            
        end % end of betaBFTest.
        
    end
    
% Public method for computing CI on c'*beta. Checked OK.
    methods (Access=public)
       
        function [CI,DF] = betaCI(slme,c,alpha,dfmethod)
%betaCI - Confidence interval on c'*beta.            
%   [CI,DF] = betaCI(slme,c,alpha,dfmethod) takes an object of type
%   StandardLinearMixedModel slme, a p by 1 vector c, a scalar alpha
%   between 0 and 1 and a string dfmethod which is one of 'none',
%   'residual' or 'satterthwaite' and computes a (1-alpha) confidence
%   interval for c'*beta. CI has two columns and 1 row. CI(1) and CI(2)
%   contain respectively the lower and upper limit of the confidence
%   interval for c'*beta. DF is the degrees of freedom used to compute the
%   confidence interval.
   
            % (1) Ensure that c is sensible.
            if size(c,1) == 1
                c = c';
            end
            assert( all(size(c) == [slme.p,1]) );
            
            % (2) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );
                        
            % (3) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );
            
            % (4) Compute the df based on dfmethod.
            switch dfmethod
                case 'none'
                    DF = Inf;
                case 'residual'
                    DF = slme.N - slme.rankX;
                case 'satterthwaite'
                    DF = dfBetaTTest(slme,c);
                otherwise
                    % <entry key="BadDFMethod">Degrees of freedom method must be ''None'', ''Residual'' or ''Satterthwaite''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadDFMethod'));                    
            end
            
            % (5) Get the covariance of betaHat.
            V = slme.covbetaHat;
            
            % (6) Compute half width of the CI.
            delta = tinv(1-alpha/2,DF)*sqrt(c'*V*c);
            
            % (7) Form the output CI.
            ctbetaHat = c'*slme.betaHat;
            CI = [ctbetaHat - delta, ctbetaHat + delta];
            
        end % end of betaCI.
        
    end
 
% Public method for computing CI on t'*beta + s'*b. Checked OK.
    methods (Access=public)
        
        function [CI,DF] = betaBCI(slme,t,s,alpha,dfmethod)
%betaBCI - Confidence interval on t'*beta + s'*b.            
%   [CI,DF] = betaBCI(slme,t,s,alpha,dfmethod) takes an object of type
%   StandardLinearMixedModel slme, a p by 1 vector t, a q by 1 vector s, a
%   scalar alpha between 0 and 1 and a string dfmethod which is one of
%   'none', 'residual' or 'satterthwaite' and computes a (1-alpha)
%   confidence interval for t'*beta + s'*b. CI has two columns and 1 row.
%   CI(1) and CI(2) contain respectively the lower and upper limit of the
%   confidence interval for t'*beta + s'*b. DF is the degrees of freedom
%   used to compute the confidence interval.
            
            % (1) Ensure that t is sensible.
            if size(t,1) == 1
                t = t';
            end
            assert( all(size(t) == [slme.p,1]) );
            
            % (2) Ensure that s is sensible.
            if size(s,1) == 1
                s = s';
            end
            assert( all(size(s) == [slme.q,1]) );
            
            % (3) Get the joint contast vector c.
            c = [t;s];
            
            % (3) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );
                        
            % (4) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );

            % (5) Compute the df based on dfmethod.
            switch dfmethod
                case 'none'
                    DF = Inf;
                case 'residual'
                    DF = slme.N - slme.rankX;
                case 'satterthwaite'
                    DF = dfBetaBTTest(slme,c);
                otherwise
                    % <entry key="BadDFMethod">Degrees of freedom method must be ''None'', ''Residual'' or ''Satterthwaite''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadDFMethod'));                    
            end
            
            % (6) Get the covariance of [betaHat-beta;bHat-b].
            V = covBetaHatBHat(slme);

            % (7) Compute half width of the CI.
            delta = tinv(1-alpha/2,DF)*sqrt(c'*V*c);
            
            % (8) Form the output CI.
            ctbetaHatbHat = c'*[slme.betaHat;slme.bHat];
            CI = [ctbetaHatbHat - delta, ctbetaHatbHat + delta];
            
        end % end of betaBCI.
        
    end
    
% Public method for computing model criterion. Checked OK.
    methods (Access=public)
       
        function crittable= modelCriterion(slme)
%modelCriterion - Compute table containing model criterion info.
%   crittable= modelCriterion(slme) takes a StandardLinearMixedModel object
%   slme and computes a table of model criterion such as AIC, BIC, logLik 
%   and Deviance.

    % Create a structure stats for classreg.regr.modelutils.modelcriterion
    % such that:
    % L = stats.LogLikelihood;
    % k = stats.NumCoefficients;
    % n = stats.NumObservations;      
    
            % (1) Get N and p.
            N = slme.N;
            p = slme.p;
            
            % (2) Add the fixed effects + residual variance to
            % NumCoefficients.
            stats.NumCoefficients = ...
                slme.Psi.NumParametersExcludingSigma + (p+1);
            
            % (3) Get effective number of observations based on FitMethod.
            switch lower(slme.FitMethod)
                case {'ml'}
                    stats.NumObservations = N;
                case {'reml'}
                    stats.NumObservations = (N-p);
                otherwise
                    % <entry key="BadFitMethod">''FitMethod'' must be either ''ML'' or ''REML''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadFitMethod'));                    
            end
            
            % (4) Set maximized log likelihood.
            loglikHat = slme.loglikHat;
            stats.LogLikelihood = loglikHat;
            
            % (5) Call modelcriterion utility function.
            crit = classreg.regr.modelutils.modelcriterion(stats,'all',true);
            
            % (6) Get deviance and create output table.
            Deviance = -2*loglikHat;
            crittable = dataset(crit.AIC, crit.BIC, loglikHat, Deviance,...
                'VarNames',{'AIC' 'BIC' 'logLik' 'Deviance'});
            
        end % end of modelCriterion.      
        
    end
    
% Public method for making table of fixed effects. Checked OK.
    methods (Access=public)
    
        function fetable = fixedEffects(slme,alpha,dfmethod)
%fixedEffects - Make a table of fixed effects.
%   fetable = fixedEffects(slme,alpha,dfmethod) takes an object slme of 
%   type StandardLinearMixedModel and makes a fixed effects table. fetable 
%   is a dataset array with the following columns:
%
%           Estimate - Estimate of the fixed effect
%           SE - Standard deviation of the estimate of fixed effect.
%           tStat - t-stat for testing the true fixed effect against 0.
%           DF - Degrees of freedom used in the t-test. 
%           pValue - p-value from the t-test.
%           Lower - Lower limit of a (1-alpha) CI.
%           Upper - Upper limit of a (1-alpha) CI.
%
%   The input alpha is a number between 0 and 1 and dfmethod is one of
%   'satterthwaite','none' or 'residual'.

            % (1) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );
                        
            % (2) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );

            % (3) Create a p by p identity matrix Ip.
            p = slme.p;
            Ip = speye(p);
            
            % (4) Get the SE column.
            se = sqrt(diag(slme.covbetaHat));
            
            % (5) Initialize P, T and DF and bounds of the confidence
            % interval LB and UB.
            P = zeros(p,1);
            T = zeros(p,1);
            DF = zeros(p,1);
            LB = zeros(p,1);
            UB = zeros(p,1);
            
            % (6) Test c'*beta = 0 and compute (1-alpha) CI for c'*beta for
            % c = Ip(:,i) and for i = 1:p.
            for i = 1:p               
                c = full(Ip(:,i));
                [P(i),T(i),DF(i)] = betaTTest(slme,c,0,dfmethod);
                CI = betaCI(slme,c,alpha,dfmethod);
                LB(i) = CI(1);
                UB(i) = CI(2);
            end
            
            % (7) Make sure that none of the DF values are NaN.
            if any(isnan(DF))
                % <entry key="ErrorDFCalculation">Degrees of freedom calculation produced a NaN result. Try using a different degrees of freedom method.</entry>
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:ErrorDFCalculation'));                
            end            
            
            % (8) Make the output dataset array.
            fetable = dataset({slme.betaHat,'Estimate'},{se,'SE'},...
                {T,'tStat'},{DF,'DF'},{P,'pValue'},...
                {LB,'Lower'},{UB,'Upper'});
            
        end % end of fixedEffects.
    
    end
  
% Public method for making table of random effects. Checked OK.
    methods (Access=public)
       
        function retable = randomEffects(slme,alpha,dfmethod)
%randomEffects - Make a table of random effects.
%   retable = randomEffects(slme,alpha,dfmethod) takes an object slme of 
%   type StandardLinearMixedModel and makes a random effects table. retable
%   is a dataset array with the following columns:
%
%           Estimate - Estimate of the random effect
%           SEPred - Standard deviation of elements of (bHat - b).
%           tStat - t-stat for testing the random effect against 0.
%           DF - Degrees of freedom used in the t-test. 
%           pValue - p-value from the t-test.
%           Lower - Lower limit of a (1-alpha) CI.
%           Upper - Upper limit of a (1-alpha) CI.
%
%   The input alpha is a number between 0 and 1 and dfmethod is one of
%   'satterthwaite','none' or 'residual'.            
            
             % (1) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );
                        
            % (2) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );

            % (3) Get p and q from slme.
            p = slme.p;
            q = slme.q;
            
            % (3) Since we don't test the fixed effects here, let t =
            % zeros(p,1).
            t = zeros(p,1);
            
            % (4) Make Iq, a q by q identity matrix.
            Iq = speye(q);
            
            % (5) Store covbetaHatbHat in slme so that calls to betaBTTest
            % are not expensive. This happens becuase covBetaHatBHat
            % calculates the covariance from scratch only if covbetaHatbHat
            % property of slme is empty.
            slme = storeCovBetaHatBHat(slme);
            %             if isempty(slme.covbetaHatbHat)
            %                 slme.covbetaHatbHat = covBetaHatBHat(slme);
            %             end
            
            % (6) Get the covariance of [betaHat-beta;bHat-b]. Retain only
            % p+1:end rows and p+1:end columns.
            V = covBetaHatBHat(slme);
            V = V(p+1:end,p+1:end);
            
            % (7) Get the SE Pred column.
            se = sqrt(diag(V));
            
            % (8) Initialize P, T and DF and bounds of the confidence
            % interval LB and UB.
            P = zeros(q,1);
            T = zeros(q,1);
            DF = zeros(q,1);
            LB = zeros(q,1);
            UB = zeros(q,1);

            % (9) Test t'*beta + s'*b = 0 (with t = 0). Compute (1-alpha) 
            % CI for s'*b for s = Iq(:,i) and for i = 1:q.
            for i = 1:q               
                s = full(Iq(:,i));
                [P(i),T(i),DF(i)] = betaBTTest(slme,t,s,0,dfmethod);
                CI = betaBCI(slme,t,s,alpha,dfmethod);
                LB(i) = CI(1);
                UB(i) = CI(2);
            end
            
            % (10) Make sure that none of the DF values are NaN.
            if any(isnan(DF))
                % <entry key="ErrorDFCalculation">Degrees of freedom calculation produced a NaN result. Try using a different degrees of freedom method.</entry>
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:ErrorDFCalculation'));                
            end            
            
            % (11) Make the output dataset array.
            retable = dataset({slme.bHat,'Estimate'},{se,'SEPred'},...
                {T,'tStat'},{DF,'DF'},{P,'pValue'},...
                {LB,'Lower'},{UB,'Upper'});            
            
        end % end of randomEffects.
        
    end
 
 % Public method to make a table of predictions (pointwise, curve). Checked OK.
    methods (Access=public)
       
        function predtable = predictTable(slme,Xnew,Znew,alpha,dfmethod)
%predictTable - Predict output from a fitted StandardLinearMixedModel.
%   predtable = predictTable(slme,Xnew,Znew,alpha,dfmethod) takes a fitted
%   StandardLinearMixedModel slme, a fixed effects design matrix Xnew, a
%   random effects design matrix Znew, a confidence level alpha, a degrees
%   of freedom method dfmethod and produces (1-alpha) pointwise confidence
%   intervals for the predictions from slme at Xnew and Znew. The
%   predictions include contributions from both fixed effects and BLUPs of
%   random effects. Xnew must be M by p and Znew must be M by q where p =
%   slme.p and q = slme.q. The output predtable is a dataset array with the
%   following columns:
%
%       Pred - The prediction
%       SEPred - SE of Prediction error
%       DF - The degrees of freedom used to compute the confidence interval
%       Alpha - The alpha value used for the confidence interval
%       Lower - Lower limit of the confidence interval
%       Upper - Upper limit of the confidence interval

            % (1) Ensure that Xnew is sensible.
            assert( isnumeric(Xnew) & isreal(Xnew) & ismatrix(Xnew) );
            assert( size(Xnew,2) == slme.p );
            
            % (2) Ensure that Znew is sensible.
            assert( isnumeric(Znew) & isreal(Znew) & ismatrix(Znew) );
            assert( size(Znew,2) == slme.q );
            
            % (3) Ensure that Xnew and Znew have the same number of rows.
            assert( size(Xnew,1) == size(Znew,1) );

            % (4) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );
                        
            % (5) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );
      
            % (6) Get the number of rows M in Xnew.            
            M = size(Xnew,1);
                       
            % (7) Get the covariance of [betaHat-beta;bHat-b].
            V = covBetaHatBHat(slme);                        
            
            % (8) Get betaHat and bHat.
            betaHat = slme.betaHat;
            bHat = slme.bHat;
            
            % (9) Get ypred.
            ypred = Xnew*betaHat + Znew*bHat;
            
            % (10) Get Alpha.
            Alpha = alpha*ones(M,1);
            
            % (11) Initialize Pred, SEPred, DF, Alpha, LB and UB.
            SEPred = zeros(M,1);
            DF = zeros(M,1);
            LB = zeros(M,1);
            UB = zeros(M,1);                       
            for i = 1:M
                % (12) Compute (1-alpha) CI for t'*beta + s'*b where t is 
                % given by the rows of Xnew and s is given by the rows of 
                % Znew. Store the results in LB, UB and DF.
                t = Xnew(i,:)';
                s = Znew(i,:)';
                [CI0,DF0] = betaBCI(slme,t,s,alpha,dfmethod);
                LB(i) = CI0(1);
                UB(i) = CI0(2);
                DF(i) = DF0;
                
                % (13) Compute sqrt(c'*V*c) where c = [t;s].
                c = [t;s];
                SEPred(i) = sqrt(c'*V*c);                
            end
            
            % (14) Make sure that none of the DF values are NaN.
            if any(isnan(DF))
                % <entry key="ErrorDFCalculation">Degrees of freedom calculation produced a NaN result. Try using a different degrees of freedom method.</entry>
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:ErrorDFCalculation'));                
            end            
            
            % (15) Make the output dataset array.
            predtable = dataset({ypred,'Pred'},{SEPred,'SEPred'},...
                {DF,'DF'},{Alpha,'Alpha'},{LB,'Lower'},{UB,'Upper'});                                    
            
        end % end of predictTable.
        
    end
    
% Public method for making a table of covariance parameters on the Canonical scale. Checked OK.
    methods (Access=public)
       
        function covtable = covarianceParameters(slme,alpha)
%covarianceParameters - Compute a table containing covariance parameters.
%   covtable = covarianceParameters(slme,alpha) takes an object slme of
%   type StandardLinearMixedModel and alpha which is a number between 0 and
%   1 and returns a dataset array with the following columns:
%
%           Estimate - Estimate of the covariance parameter 
%           Lower - Lower bound of (1-alpha) CI for the covariance parameter
%           Upper - Upper bound of (1-alpha) CI for the covariance parameter
%            
%   The last row of the dataset corresponds to the residual standard deviation. 

            % (1) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );
            
            % (2) Get the estimate of canonical parameters heta.
            hetaHat = slme.Psi.getCanonicalParameters;
            
            % (3) Get the estimate of sigma.
            sigmaHat = slme.sigmaHat;
            
            % (4) Form x = [hetaHat;sigmaHat].
            x = [hetaHat;sigmaHat];

            % (5) Create output covtable with just the 'Estimate'.
            covtable = dataset({x,'Estimate'});
            
            % (6) Get (1-alpha) CI for [heta;sigma]. This may not work
            % sometimes because the covariance matrix of [etaHat;log(sigmaHat)]
            % is not positive definite. When this happens we error out.            
            try                
                CI = hetaSigmaCI(slme,alpha);
                covtable.Lower = CI(:,1);
                covtable.Upper = CI(:,2);
            catch ME
                % <entry key="Message_TooManyCovarianceParameters">Cannot compute confidence intervals on covariance parameters. The model appears to contain many covariance parameters not all of which are uniquely identifiable. Try to simplify the covariance structure and refit the model.</entry>
                msgID = 'stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_TooManyCovarianceParameters';
                msgStr = getString(message(msgID));
                baseME = MException(msgID,msgStr);
                ME = addCause(baseME,ME);
                warning('stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_TooManyCovarianceParameters',ME.message);
            end                                    
                        
        end % end of covarianceParameters.
        
    end

% Public method to get estimated posterior covariance of random effects vector. Checked OK.
    methods (Access=public)
        
        function C = postCovb(slme)
%postCovb - Posterior covariance of b. 
%   C = postCovb(slme) takes a StandardLinearMixedModel object slme and
%   computes the estimated posterior covariance matrix of b.

            % The posterior covariance is given by:
            % C = sigma^2 * inv(Z'*Z + Delta'*Delta) 
            % with Delta'*Delta = inv(D).
            % We replace sigma by sigmaHat.

            % (1) Get Lambda = inv(Delta).
            Lambda = getLowerTriangularCholeskyFactor(slme.Psi);
            
            % (2) Get Z and q.
            Z = slme.Z;
            q = slme.q;
            
            % (3) Form the matrix U.
            U = Z*Lambda;
            
            % (4) Get R and S such that S*R'*R*S' = U'*U + eye(q) where R
            % and S are q by q. S is a permutation matrix: S*S' = eye(q).
            Iq = spdiags(ones(q,1),0,q,q);
            [R,status,S] = chol(U'*U + Iq);
            
            % (5) Ensure that Cholesky factorization worked.
            if ( status ~= 0 )
                % <entry key="ErrorPostCovb">An error occured in getting the posterior covariance of random effects vector.</entry>
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:ErrorPostCovb'));
            end
            
            % (6) Get sigmaHat.
            sigmaHat = slme.sigmaHat;
            
            % (7) C can also be written as:
            %     C = sigma^2 * (Lambda*S*inv(R) * inv(R)'*S'*Lambda').
            % We replace sigma by sigmaHat.
            T = (Lambda*S) / R;
            C = (sigmaHat^2) * (T*T');
            
        end % end of postCovb.
    
    end
    
% Public methods related to fitted values. Checked OK.
    methods (Access=public)
        
        function yfit = fitted(slme,wantConditional)
%fitted - Returns the fitted response from a StandardLinearMixedModel.
%   yfit = fitted(slme,true) returns the N by 1 vector representing the
%   fitted conditional response from the StandardLinearMixedModel slme such
%   that yfit = X*betaHat + Z*bHat.
%
%   yfit = fitted(slme,false) returns the N by 1 vector representing the
%   fitted marginal response from the StandardLinearMixedModel slme such
%   that yfit = X*betaHat.

            % (1) Ensure that wantConditional is sensible.
            assert( islogical(wantConditional) & isscalar(wantConditional) );

            % (2) Decide which type of fitted values to return.
            if ( wantConditional == true )
                yfit = slme.X*slme.betaHat + slme.Z*slme.bHat;
            else
                yfit = slme.X*slme.betaHat;
            end

    end % end of fitted.                                                
       
    end
 
% Public method to make predictions on new data.
    methods (Access=public)
       
        function [ypred,CI,DF] = predict(slme,Xnew,Znew,alpha,dfmethod,wantConditional,wantPointwise,wantCurve,hasIntercept)
%predict - Predictions from a fitted StandardLinearMixedModel.
%   [ypred,CI,DF] = predict(slme,Xnew,Znew,alpha,dfmethod,wantConditional,wantPointwise,wantCurve,hasIntercept)
%   takes a fitted StandardLinearMixedModel slme, a fixed effects design
%   matrix Xnew, a random effects design matrix Znew, a confidence level
%   alpha, a degrees of freedom method dfmethod, flags for conditional,
%   pointwise and curve predictions and outputs predictions and (1-alpha)
%   confidence intervals (CIs) for the true predictions from slme at Xnew
%   and Znew. If wantConditional is true then the predictions include
%   contributions from both fixed effects and BLUPs of random effects
%   otherwise the predictions include contributions from only the fixed
%   effects. If wantPointwise is true then pointwise CIs are computed
%   otherwise simultaneous CIs are computed. If wantCurve is true then CIs
%   are for the curve excluding variability due to observation noise
%   otherwise variability due to observation noise is included. Xnew must
%   be M by p and Znew must be M by q where p = slme.p and q = slme.q. The
%   output ypred is a M by 1 vector containing the predictions
%   corresponding to the rows of Xnew and Znew and output CI is a M by 2
%   matrix. Each row of CI is a (1-alpha) confidence interval for the true
%   prediction such that CI(:,1) contains the lower confidence limit and
%   CI(:,2) contains the upper confidence limit. DF is a M by 1 vector
%   containing the DF values used in computing the CIs if wantPointwise is
%   true. If wantPointwise is false then DF is a scalar containing the DF
%   value used for computing the Scheffe simultaneous CIs. hasIntercept
%   should be set to true if the fixed effects part of the model has an
%   intercept and false otherwise.
            
            % (1) Ensure that Xnew is sensible.
            assert( isnumeric(Xnew) & isreal(Xnew) & ismatrix(Xnew) );
            assert( size(Xnew,2) == slme.p );
            
            % (2) Ensure that Znew is sensible.
            assert( isnumeric(Znew) & isreal(Znew) & ismatrix(Znew) );
            assert( size(Znew,2) == slme.q );
            
            % (3) Ensure that Xnew and Znew have the same number of rows.
            assert( size(Xnew,1) == size(Znew,1) );

            % (4) Ensure that alpha is sensible.
            assert( isnumeric(alpha) & isreal(alpha) & isscalar(alpha) );
            assert( alpha >= 0 & alpha <= 1 );
                        
            % (5) Ensure that dfmethod is one of 'none','residual' or
            % 'satterthwaite'.
            dfmethod = lower(dfmethod);
            assert( ischar(dfmethod) & isvector(dfmethod) ...
                & (size(dfmethod,1) == 1) );
            assert( any(strcmpi(dfmethod,slme.AllowedDFMethods)) );

            % (6) Ensure that wantConditional is a scalar logical.
            assert( isscalar(wantConditional) & islogical(wantConditional) );
            
            % (7) Ensure that wantPointwise is a scalar logical.
            assert( isscalar(wantPointwise) & islogical(wantPointwise) );
            
            % (8) Ensure that wantCurve is a scalar logical.
            assert( isscalar(wantCurve) & islogical(wantCurve) );
            
            % (9) Ensure that hasIntercept is a scalar logical.
            assert( isscalar(hasIntercept) & islogical(hasIntercept) );
            
            % (10) Get conditional or marginal prediction ypred.
            if ( wantConditional == true )
                ypred = Xnew*slme.betaHat + Znew*slme.bHat;                                
            else
                ypred = Xnew*slme.betaHat;                
            end
            
            if nargout > 1                
                % (11) Get a vector of estimated variance of predictions. 
                % To do this form matrices C and V such that:
                %
                % (a) For marginal predictions:
                %       C = [Xnew] and V = slme.covbetaHat 
                % (b) For conditional predictions:
                %       C = [Xnew,Znew] and V = covBetaHatBHat(slme);
                % 
                % The ith element of the vector of estimated prediction 
                % variances is then given by: 
                % 
                % varpred(i) = C(i,:)*V*C(i,:)'
                %
                % Add variability due to observation noise to varpred if
                % wantCurve is false.                
                if ( wantConditional == true ) 
                    C = [Xnew,Znew];
                    V = covBetaHatBHat(slme);                    
                else
                    C = Xnew;
                    V = slme.covbetaHat;                    
                end
                varpred = sum(C .* (C*V'),2);
                
                % (12) If wantCurve is false, add residual noise variance 
                % slme.sigmaHat^2 to each element of varpred.
                if ( wantCurve == false )
                    varpred = varpred + (slme.sigmaHat)^2;
                end                
                
                % (13) Get the number of rows in Xnew, the effective number
                % of columns in slme.X and the number of columns in slme.Z.
                M = size(Xnew,1);
                rankX = slme.rankX;
                q = slme.q;
                
                % (14) Compute elementwise multiplier crit of sqrt(varpred) 
                % depending on whether wantPointwise is true or false.
                if ( wantPointwise == true )
                    % (1) Get a vector of DF values.
                    switch dfmethod
                        case 'none'
                            DF = Inf*ones(M,1);
                        case 'residual'
                            DF = (slme.N - rankX)*ones(M,1);
                        case 'satterthwaite'
                            DF = zeros(M,1);
                            for i = 1:M
                                if ( wantConditional == true )
                                    DF(i) = dfBetaBTTest(slme,C(i,:));
                                else
                                    DF(i) = dfBetaTTest(slme,C(i,:));
                                end
                            end
                        otherwise
                            % <entry key="BadDFMethod">Degrees of freedom method must be ''None'', ''Residual'' or ''Satterthwaite''.</entry>
                            error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadDFMethod'));                            
                    end
                    % (2) Compute multiplier vector crit.
                    crit = tinv(1-alpha/2,DF);
                else
                    % (1) DF will be a scalar.                    
                    switch dfmethod
                        case 'none'
                            DF = Inf;
                        case 'residual'
                            DF = (slme.N - rankX);
                        case 'satterthwaite'
                            if ( wantConditional == true )
                                DF = dfBetaBFTest(slme,eye(rankX + q));
                            else
                                DF = dfBetaFTest(slme,eye(rankX));
                            end
                        otherwise
                            % <entry key="BadDFMethod">Degrees of freedom method must be ''None'', ''Residual'' or ''Satterthwaite''.</entry>
                            error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadDFMethod'));                            
                    end
                    
                    % (2) Set numerator DF for Scheffe multiplier. Add 1 to
                    % numerator df if wantCurve is false (observation).
                    if ( wantConditional == true )
                        if ( wantCurve == true )
                            numDF = rankX + q;
                        else
                            if hasIntercept
                                numDF = rankX + q;
                            else
                                numDF = rankX + q + 1;
                            end
                        end
                    else
                        if ( wantCurve == true )
                            numDF = rankX;
                        else
                            if hasIntercept
                                numDF = rankX;
                            else
                                numDF = rankX + 1;
                            end
                        end
                    end
                    
                    % (3) Compute crit for simultaneous inference.
                    crit = sqrt( numDF*finv(1-alpha,numDF,DF) );                    
                end
                
                % (15) Compute the CIs.
                delta = crit.*sqrt(varpred);
                CI = [ypred - delta, ypred + delta];            
            end            
            
        end % end of predict.
                
    end
    
% Public method for generating random data from the fitted model. Checked OK.
    methods (Access=public)
        
        function bsim = randomb(slme,S,NumReps)
%randomb - Generate random effects vectors using a fitted model.           
%   bsim = randomb(slme,S,NumReps) takes a fitted StandardLinearMixedModel
%   slme and generates random effects vectors in bsim from slme using the 
%   RandStream object S. If slme.Psi has K blocks then NumReps is a length
%   K vector. The vector bsim is a column vector:
%
%   bsim = [b(1)_1;b(1)_2;...;b(1)_NumReps(1);...
%           b(2)_1;b(2)_2;...;b(2)_NumReps(2);...
%           b(K)_1;b(K)_2;...;b(K)_NumReps(K)]
%   where
%       b(i)_j = jth vector from ith matrix slme.Psi.Matrices{i}.
%
%   For block i, we generate NumReps(i) independent random vectors, each of
%   size Psi.SizeVec(i) from slme.Psi.Matrices{i} and concatenate them
%   vertically. We do this for all the blocks and return a concatenated
%   vector of size sum_{i = 1 to K} NumReps(i) * Psi.SizeVec(i).

            % (1) Extract Psi from slme. 
            Psi = slme.Psi;
                        
            % (2) Also extract SizeVec and NumBlocks from Psi.
            SizeVec = Psi.SizeVec;
            NumBlocks = Psi.NumBlocks;
            
            % (3) NumReps is a numeric, real, vector of length NumBlocks.
            assert( isnumeric(NumReps) & isreal(NumReps) ...
                & isvector(NumReps) & length(NumReps) == NumBlocks );
            
            % (4) Make NumReps into a column vector.
            if size(NumReps,1) == 1
                NumReps = NumReps';
            end
            
            % (5) Psi is a BlockedCovariance object. It has NumBlocks
            % smaller covariance matrices given in the Psi.Matrices cell
            % array. Psi.Matrices{i} has size SizeVec(i).
            
            % (6) The combined random effects vector bsim will be of size 
            % sum_{i=1 to NumBlocks} SizeVec(i)*NumReps(i).            
            bsim = zeros(sum(SizeVec .* NumReps),1);
                        
            % (7) Loop over NumBlocks. For block i, generate NumReps(i)
            % vectors from Psi.Matrices{i}, the length of each such vector
            % being SizeVec(i) times 1.
            if ~isempty(bsim)
                offset = 0;
                for r = 1:NumBlocks       
                    % (8) Get the covariance matrix PSIr for block r.
                    Lr = getLowerTriangularCholeskyFactor(Psi.Matrices{r});
                    sigmar = getSigma(Psi);
                    PSIr = (sigmar^2)*(Lr*Lr');           

                    % (9) Get the row vector for the mean of block r.
                    mur = zeros(1,SizeVec(r));

                    % (10) How many vectors to be drawn for the current block?
                    Nr = NumReps(r);

                    % (11) Generate random vectors in columns of temp.
                    if isempty(S)
                       % Use default mvnrnd to get a matrix with Nr columns.
                       temp = mvnrnd(mur,PSIr,Nr)';
                       % The following call should also give the same result:
                       % temp = mymvnrnd([],mur,PSIr,Nr)';                                       
                    else
                       % Use mymvnrnd that accepts RandStream S. Get a matrix
                       % with Nr columns.
                       temp = mymvnrnd(S,mur,PSIr,Nr)';
                    end

                    % (12) Concatenate temp to get a SizeVec(r)*NumReps(r) by 1
                    % vector and store in bsim starting from offset.
                    bsim(offset+1 : offset+SizeVec(r)*NumReps(r)) = temp(:);

                    % (13) Update offset for the next block.
                    offset = offset + SizeVec(r)*NumReps(r);                
                end
            end

        end % end of randomb.
        
        function ysim = random(slme,S,Xsim,Zsim)
%random - Generate random data from a fitted StandardLinearMixedModel.
%   ysim = random(slme,S,Xsim,Zsim) takes a fitted StandardLinearMixedModel
%   slme and generates random data from slme using the RandStream object S
%   at the specified fixed effects design matrix Xsim and random effects
%   design matrix Zsim. Xsim must be M by p and Zsim must be M by q where p
%   = slme.p and q = slme.q. The output ysim is a M by 1 vector. Set S = []
%   to use the default RandStream object.
    
            % (1) Ensure that Xsim is sensible.
            assert( isnumeric(Xsim) & isreal(Xsim) & ismatrix(Xsim) );
            assert( size(Xsim,2) == slme.p );
            
            % (2) Ensure that Zsim is sensible.
            assert( isnumeric(Zsim) & isreal(Zsim) & ismatrix(Zsim) );
            assert( size(Zsim,2) == slme.q );
            
            % (3) Ensure that Xsim and Zsim have the same number of rows.
            assert( size(Xsim,1) == size(Zsim,1) );
            
            % (4) Extract Psi, betaHat, sigmaHat from slme. 
            Psi = slme.Psi;
            betaHat = slme.betaHat;
            sigmaHat = slme.sigmaHat;
            
            % (5) Also extract NumReps from Psi.
            NumReps = Psi.NumReps;

            % (6) Psi is a BlockedCovariance object. It has NumBlocks smaller
            % covariance matrices given in the Psi.Matrices cell array.
            % Psi.Matrices{i} is repeated NumReps(i) times along the block
            % diagonal of Psi. Psi.Matrices{i} has size SizeVec(i).
            
            % (7) The combined random effects vector bsim will be of size 
            % sum_{i=1 to NumBlocks} SizeVec(i)*NumReps(i).
            bsim = randomb(slme,S,NumReps);            
                       
            % (8) Generate the noise term epsilon ~ N(0,sigma^2 I_N). 
            N = size(Xsim,1);
            if isempty(S)
                % Use default RandStream.
                epsilonsim = sigmaHat*randn(N,1);
            else
                % Use user supplied RandStream S.
                epsilonsim = sigmaHat*randn(S,N,1);
            end
            
            % (9) Generate simulated data.
            if isempty(bsim)
                % Only fixed effects.
                ysim = Xsim * betaHat + epsilonsim;
            else
                ysim = Xsim * betaHat + Zsim * bsim + epsilonsim;
            end

        end % end of random.
        
    end
    
% Private method to get raw residuals (marginal and conditional). Checked OK.
    methods (Access=private)
       
        function rawr = getRawResiduals(slme,wantConditional)
%getRawResiduals - Get Raw residuals.
%   rawr = getRawResiduals(slme,wantConditional) takes an object slme of
%   type StandardLinearMixedModel and returns the raw conditional residuals
%   if wantConditional is true and the raw marginal residuals if the flag
%   wantConditional is false.
           
            % (1) Ensure that wantConditional is sensible.
            assert( isscalar(wantConditional) & islogical(wantConditional) );
            
            % (2) Decide which type of raw residuals to return.
            if ( wantConditional == true )
                % Conditional residuals. 
                rawr = slme.y - slme.X*slme.betaHat - slme.Z*slme.bHat; 
            else
                % Marginal residuals. 
                rawr = slme.y - slme.X*slme.betaHat;
            end

        end % end of getRawResiduals.
        
    end
 
% Private method to get Pearson residuals (marginal and conditional). Checked OK.
    methods (Access=private)
       
        function pearsonr = getPearsonResiduals(slme,wantConditional)
%getPearsonResiduals - Get Pearson residuals.           
%   pearsonr = getPearsonResiduals(slme,wantConditional) takes an object
%   slme of type StandardLinearMixedModel and returns the Pearson
%   conditional residuals if wantConditional is true and the Pearson
%   marginal residuals if wantConditional is false.

            % (1) Ensure that wantConditional is sensible.
            assert( isscalar(wantConditional) & islogical(wantConditional) );

            % (2) Decide which type of Pearson residuals to return.
            if ( wantConditional == true )
                % Conditional residuals.
                
                % (1) Get raw conditional residuals.
                rawr = getRawResiduals(slme,true);
                
                % (2) Divide rawr elementwise by a vector with all elements
                % equal to slme.sigmaHat: stdvec = slme.sigmaHat*ones(slme.N,1);
                stdvec = slme.sigmaHat;
                
                % (3) Form the conditional, Pearson residuals.
                pearsonr = rawr ./ stdvec;
                
            else
                % Marginal residuals.
                
                % (1) Get raw marginal residuals.
                rawr = getRawResiduals(slme,false);
                
                % Divide rawr elementwise by the sqrt of the diagonal of M
                % where:
                %
                % M = sigma^2 * inv(H) where H is a matrix such that
                %
                % inv(H) = Z*D*Z' + IN and IN = eye(N). 
                %
                % If D = Lambda*Lambda' and U = Z*Lambda then we can write:
                % 
                % inv(H) = U*U' + IN. 
                %
                % So the M matrix is: 
                % 
                % M = sigma^2 * inv(H) = sigma^2 * (U*U' + IN)
                
                % (2) Get sigmaHat and Lambda.
                sigmaHat = slme.sigmaHat;
                Lambda = getLowerTriangularCholeskyFactor(slme.Psi);
                
                % (3) Form the U matrix.
                U = slme.Z * Lambda;    
                
                % (4) Get diagonal of invH and the appropriate std vector. 
                diaginvH = ( sum(U.^2,2) + 1 );
                stdvec = sigmaHat * sqrt(diaginvH);                
                
                % (5) Form the marginal, Pearson residuals.
                pearsonr = rawr ./ stdvec;                
            end
            
        end % end of getPearsonResiduals.        
        
    end
    
% Private method to get internally Studentized residuals (marginal and conditional).
% These are called 'Standardized' residuals in LinearModel. Checked OK.
    methods (Access=private)
       
        function studr = getStudentizedResiduals(slme,wantConditional)
%getStudentizedResiduals - Get Studentized residuals.
%   studr = getStudentizedResiduals(slme,wantConditional) takes an object
%   slme of type StandardLinearMixedModel and returns the Studentized
%   conditional residuals if wantConditional is true and the Studentized
%   marginal residuals if wantConditional is false.

            % (1) Ensure that wantConditional is sensible.
            assert( isscalar(wantConditional) & islogical(wantConditional) );

            % (2) Get sigmaHat, X, Z, q, Lambda and U.
            sigmaHat = slme.sigmaHat;
            X = slme.X;
            Z = slme.Z;
            q = slme.q;
            Lambda = getLowerTriangularCholeskyFactor(slme.Psi);
            U = Z*Lambda;
            
            % (3) Get R and S such that S*R'*R*S' = U'*U + eye(q) where
            % R and S are q by q. S is a permutation matrix: S*S' = eye(q).
            Iq = spdiags(ones(q,1),0,q,q);
            [R,status,S] = chol(U'*U + Iq);
            
            % (4) Ensure that Cholesky factorization worked.
            if ( status ~= 0 )
                % <entry key="ErrorStandardizedResiduals">An error occured in getting standardized residuals.</entry> 
                error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:ErrorStandardizedResiduals'));                
            end
            
            % (5) Write inv(Z'*Z + Delta'*Delta) = T*T' where T is such
            % that T = Lambda*S*inv(R).
            T = Lambda * S / R;
            
            % (6) Write
            % X'*H*X = X'*X - X'*Z*inv(Z'*Z + Delta'*Delta)*Z'*X
            %        = X'*X - X'*Z*T*T'*Z'*X
            %        = R1*R1' where R1 is lower triangular.
            try
                Q = X'*Z*T;
                R1 = chol(X'*X - Q*Q','lower');
            catch ME %#ok<NASGU>
                R1 = classreg.regr.lmeutils.covmats.CovarianceMatrix.singularLowerChol(X'*X - Q*Q');
            end
                        
            % (7) Decide which type of Studentized residuals to return.
            if ( wantConditional == true )
                % Conditional residuals.
               
                % (1) Get the raw conditional residuals.
                rawr = getRawResiduals(slme,true);
                
                % We need to divide rawr elementwise by the sqrt of the
                % diagonal of the matrix M where 
                %
                % M = sigma^2 * K * F *K = sigma^2 * H*F*H where
                %
                % H = IN - Z*inv(Z'*Z + Delta'*Delta)*Z' and
                % F = inv(H) - X*inv(X'*H*X)*X'
                %
                % In terms of factors T and R1 (described above), we can
                % write:
                %
                % M = sigma^2 * (H - H*X*inv(X'*H*X)*X'*H)   
                %   = sigma^2 * (IN - Z*inv(Z'*Z + Delta'*Delta)*Z' - H*X*inv(X'*H*X)*X'*H)  
                %   = sigma^2 * (IN - Z*T*T'*Z' - H*X*inv(R1*R1')*X'*H) 
                %   = sigma^2 * (H*F*H)
                                
                % (2) Get H*X.
                HX = X - (Z*T)*(T'*Z'*X);
                
                % (3) Get the diagonal elements of H*F*H.
                diagHFH = 1 - sum((Z*T).^2,2) - sum((HX / R1').^2,2);
                
                % (4) Get the appropriate std vector.
                stdvec = sigmaHat*sqrt(diagHFH);
                
                % (5) Form the conditional Studentized residuals.
                studr = rawr ./ stdvec;                                                                                                
                
            else
                % Marginal residuals.
                
                % (1) Get the raw marginal residuals.
                rawr = getRawResiduals(slme,false);
                
                % We need to divide rawr elementwise by the sqrt of the
                % diagonal of the matrix M where 
                %
                % M = sigma^2 * F   where
                %
                % H = IN - Z*inv(Z'*Z + Delta'*Delta)*Z' and
                % F = inv(H) - X*inv(X'*H*X)*X'
                %
                % inv(H) = Z*D*Z' + IN where D = Lambda*Lambda'
                %
                % In terms of factors U and R1 (described above), we can
                % write:
                %
                % F = U*U' + IN - X*inv(R1*R1')*X'
                
                % (2) Get diagonal of F.
                diagF = 1 + sum((U.^2),2) - sum((X/R1').^2,2);
                
                % (3) Get the appropriate std vector.
                stdvec = sigmaHat*sqrt(diagF);
                
                % (4) Form the marginal Studentized residuals.
                studr = rawr ./ stdvec;                                                
            end
                        
        end % end of getStudentizedResiduals.
        
    end
    
% Public method to get various residual types (convenience function). Checked OK.    
    methods (Access=public)
        
        function r = residuals(slme,wantConditional,residualType)
%residuals - Convenience function to get residuals of various types.
%   r = residuals(slme,wantConditional,residualType) takes an object slme
%   of type StandardLinearMixedModel and returns the conditional residuals
%   if wantConditional is true and marginal residuals if wantConditional is
%   false. residualType is a string indicating the type of residuals
%   required. residualType can be either 'Raw', 'Pearson', 'Standardized'.
%   Note that 'Standardized' will give the internally studentized
%   residuals.

            % (1) Ensure that wantConditional is sensible.
            assert( isscalar(wantConditional) & islogical(wantConditional) );
            
            % (2) Ensure that residualType is one of 'Raw', 'Pearson' or
            % 'Standardized'.           
            residualType = lower(residualType);
            assert( ischar(residualType) & isvector(residualType) ...
                & (size(residualType,1) == 1) );
            assert( any(strcmpi(residualType,slme.AllowedResidualTypes)) );
            
            % (3) Get the desired type of residual.
            switch residualType                
                case 'raw'
                    r = getRawResiduals(slme,wantConditional);
                case 'pearson'
                    r = getPearsonResiduals(slme,wantConditional);
                case 'standardized'
                    % 'standardized' really means internally studentized.
                    % This is what getStudentizedResiduals returns.
                    r = getStudentizedResiduals(slme,wantConditional);
                otherwise 
                    % <entry key="BadResidualType">Residual type must be ''Raw'', ''Standardized'' or ''Pearson''.</entry>
                    error(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:BadResidualType'));                                        
            end            

        end % end of residuals.
        
    end

    
%     methods (Static)
%        function slme = loadobj(obj)
%           keyboard;
%        end
%     end    
    
end

%=== helper function logAbsDetTriangular
function x = logAbsDetTriangular(R)
%logAbsDetTriangular - Log absolute determinant of triangular matrix.
%   x = logAbsDetTriangular(R) takes an upper triangular or lower
%   triangular matrix R and computes the output x = log( abs(det(R)) ). R
%   is assumed to be triangular, we do not explicitly check this in this
%   method. If R is not triangular, the result will be incorrect.

%   Note: If R is of size p by p then det(R) = r_11 * r_22 * ... * r_pp and
%   so abs(det(R)) = abs(r_11) * abs(r_22) * ... * abs(r_pp). Therefore,
%   log( abs(det(R)) ) 
%       = log( abs(r_11) * abs(r_22) * ... * abs(r_pp) )
%       = log( abs(r_11) ) + log( abs(r_22) ) + ... + log( abs(r_pp) )
%       = sum_{i=1}^p log( abs(r_ii) )
%       = sum(log(abs( diag(R) )));

    x = sum(log(abs( diag(R) )));
    
end % end of logAbsDetTriangular.

%=== helper function getGradient
function g = getGradient(fun,theta,step)
%getGradient - Get the numerical gradient by finite differences.
%   g = getGradient(fun,theta,step) gets the numerical gradient using
%   central differences of the function fun evaluated at p by 1 vector
%   theta. The output g will be a column vector of size p by 1. fun is a
%   function handle to a function that accepts a vector theta and returns a
%   scalar.

    % Set step size.
    if (nargin == 2)
        step = eps^(1/3);
    end
    
    % Initialize output.
    p = length(theta);
    g = zeros(p,1);   
    for i = 1:p   
        % Use central differences.
        theta1 = theta;
        theta1(i) = theta1(i) - step;
        
        theta2 = theta;
        theta2(i) = theta2(i) + step;
        
        g(i) = (fun(theta2) - fun(theta1))/2/step;        
    end
    
end % end of getGradient.

%=== helper function getHessian
function H = getHessian(fun,theta,wantRegularized)
%getHessian - Get the numerical Hessian by finite differences.
%   H = getHessian(fun,theta,wantRegularized) gets the numerical Hessian by
%   central differences of the function fun evaluated at p by 1 vector
%   theta. The output H will be a p by p matrix. fun is a function handle
%   to a function that accepts a vector theta and returns a scalar. If
%   wantRegularized is true then we return H + deltaH where deltaH is given
%   by deltaH = delta*eye(size(H)), delta = abs(min(eig(H))) + sqrt(eps).
%   The idea is that H may have negative eigenvalues but H + deltaH is
%   positive definite. The default value of wantRegularized is false.

    % Set step size.
    step = eps^(1/4);
    
    % Initialize output.
    p = length(theta);    
    H = zeros(p,p);
    for i = 1:p
        % Use central differences.
        theta1 = theta;
        theta1(i) = theta1(i) - step;
        
        theta2 = theta;
        theta2(i) = theta2(i) + step;
        
        H(:,i) = ( getGradient(fun, theta2, step) -  getGradient(fun, theta1, step) )/2/step;       
    end
    
    if ( nargin == 3 && wantRegularized )        
        lambda = eig(H);
        if ~all(lambda > 0)
            % There is a -ve or zero eigenvalue.
            delta = abs(min(lambda)) + sqrt(eps);
            % Add a multiple of identity to H.
            H = H + delta*eye(size(H));
        end
    end
    
end % end of getHessian.

%=== helper function mymvnrnd
function R = mymvnrnd(S, MU, SIGMA, N)
% This helper function works exactly like mvnrnd, except that it uses a
% specified RandStream object to generate samples.
%
% INPUTS:
%
%     S = A RandStream object from which to draw samples. This can be empty
%         in which case, the default RandStream will be used.
%    MU = 1 by p mean vector of the multivariate Normal distribution.
% SIGMA = p by p co-variance matrix of the multivariate Normal distribution.
%     N = number of 1 by p vectors to sample from the multivariate Normal distribution.
%
% OUTPUTS:
%     R = N by p matrix where each row is a draw from a multivariate Normal
%         distribution with mean vector MU and co-variance matrix SIGMA.
% 
%
% Suppose x is a p by 1 vector with x ~ N(mu, sigma) where mu is a p by 1
% mean vector and sigma is a p by p co-variance matrix. The function cholcov 
% computes a factor T such that:
% 
% T = cholcov(sigma) and 
% sigma = T'*T.
%
% Now,
%        x ~ N(mu, sigma) 
%
%       x - mu ~ N(0, sigma)
%
%     inv(T')*(x-mu) ~ N(0, I_p)
%
% Suppose y ~ N(0, I_p) is a p by 1 vector (using randn for example) then
%
% inv(T')*(x-mu) = y
%
% Thus,  x = mu + T'*y
%
%        x' = mu' + y' * T
%
% If we want N vectors x_1,x_2,..,x_N from N(mu, sigma) then:
%
%       [x_1'       [mu'     [y_1'
%        x_2'        mu'      y_2'
%        x_3'    =   mu'  +   y_3'   * T
%         .           .        .
%         .           .        .
%        x_N']       mu']     y_N']
%
% The N by p matrix formed from y_1,y_2...,y_N can be generated using
% randn(N,p). Also note that the input to our function MU = mu'. Hence we
% can generate the N by p matrix made up of x_1,...,x_N (which is the same
% as the output of our function R) as follows;
%
%       [x_1'       
%        x_2'        
%        x_3'    =  R = N by p matrix = repmat(MU,N,1) + randn(N,p) * T; 
%         .           
%         .           
%        x_N']      
%

    % Get p and make sure MU and SIGMA are of the right size.
    p = size(MU,2);
    assert(p == size(SIGMA,1));
    assert(p == size(SIGMA,2));

    % Compute the factor T.
    T = cholcov(SIGMA);
    
    % Compute the output matrix R.
    if isempty(S)
        % Use default RandStream.
        R = bsxfun(@plus, MU, randn(N,p)*T);
    else
        % Use user supplied RandStream.
        assert( isa(S,'RandStream') );
        R = bsxfun(@plus, MU, randn(S,N,p)*T);
    end
    
end % end of mymvnrnd.

%=== helper function checkPositiveDefinite
function checkPositiveDefinite(H,msgID,varargin)
%checkPositiveDefinite - Checks if the input matrix is positive definite.
%   checkPositiveDefinite(H,msgID,varargin) takes a square matrix H that is
%   expected to be symmetric positive definite but may have negative
%   eigenvalues due to numerical roundoff. If a Cholesky factorization of H
%   fails, a warning is displayed to the user with the content given by the
%   message ID msgID. varargin can contain additional arguments required to
%   construct a message with ID msgID.
         
        try 
            chol(H);
        catch ME %#ok<NASGU> 
            % (1) Construct a message object and extract its string content.
            msg = message(msgID,varargin{:});
            msgStr = getString(msg);            
            
            % (2) Does H have any NaN's or Inf's?
            hasNaN = any(isnan(H(:)));
            hasInf = any(isinf(H(:)));           
            
            % (3) Compute the smallest eigenvalue of H if it has no NaN or
            % Inf values. Otherwise, say that H has Inf/NaN values.
            if ~hasNaN && ~hasInf
                mineigH = min(eig(H));
                % <entry key="Message_MinEig">Minimum eigenvalue is {0}.</entry>
                eigmsgStr = getString(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_MinEig',num2str(mineigH)));
                msgStr = [msgStr,' ',eigmsgStr];
            else
                % <entry key="Message_NaNInfInHessian">Hessian has NaNs or Infs.</entry>
                naninfmsgStr = getString(message('stats:classreg:regr:lmeutils:StandardLinearMixedModel:Message_NaNInfInHessian'));
                msgStr = [msgStr,' ',naninfmsgStr];
            end
            % (4) Throw a warning with the message ID = msg.Identifier.
            warning(msg.Identifier,msgStr);
        end
        
end % end checkPositiveDefinite.

%=== helper function hasNaNInf
function tf = hasNaNInf(X)
%hasNaNInf - Check if input has NaNs or Infs.
%   tf = hasNaNInf(X) takes a matrix/vector X and if X has NaNs or Infs 
%   anywhere in it then tf is true otherwise tf is false.

    tf = any( isnan(X(:)) | isinf(X(:)) );

end % end of hasNaNInf. 

function [ok,p,H,C]=fullrankH(H,C)
%FULLRANKH Make sure hypothesis matrix has full rank.
% [ok,p,H,C]=fullrankH(H,C) accepts a M by q matrix H and a M by 1 vector C
% used for testing the null hypothesis H*beta = C and returns a full row
% rank matrix H and a vector C that can be used for testing H*beta = C. ok
% is true if the input H and C are consistent otherwise it is false. p is 
% the rank of the input H.

% NOTE: The matrix H may not be of full row rank but H and C together must
% be sensible. For example, if beta is a 3 by 1 vector then H = [1 0 0] and
% C = [0] express the null hypothesis beta(1) = 0. The same hypothesis can
% be expressed by choosing H = [1 0 0;1 0 0] and C = [0 0] in which case
% the null hypothesis becomes [beta(1);beta(1)] = [0;0]. The second
% equation is redundant but the set of equations is still consistent. On
% the other hand, H = [1 0 0;1 0 0] and C = [1;0] express the null
% hypothesis [beta(1);beta(1)] = [1;0]. Clearly, these equations are not
% consistent.

    % (1) Find the rank of H and a basis for the row space.
    [~,r,e] = qr(H',0);
    tol = eps(norm(H))^(3/4);
    p = sum(abs(diag(r)) > max(size(H))*tol*abs(r(1,1)));

    % (2) Get a list of linearly dependent contrasts.
    E = e(1:max(1,p));

    % (3) Find coefficients that satisfy these contrasts.
    b = H(E,:)\C(E);

    % (4) Make sure they satisfy the entire set of contrasts.
    ok = all(abs(H*b-C) < tol);

    % (5) Now it is sufficient to use a full-rank subset.
    H = H(E,:);
    C = C(E);
    
end % end of fullrankH.

%=== helper method for verifying assertions
function assertThat(condition,msgID,varargin)
%   assertThat(condition,msgID,varargin) takes a variable condition that is
%   either true or false, a message catalog ID msgID and optional arguments
%   required to create a message object from msgID. If condition is false,
%   an error message represented by msgID is thrown.

    if ~condition        
        % (1) Create a message object from msgID and varargin.
        try
            msg = message(msgID,varargin{:});
        catch
            % <entry key="BadMsgID">Invalid message ID: {0}.</entry>      
            error(message('stats:LinearMixedModel:BadMsgID',msgID));
        end        
        % (2) Create and throw an MException.
        ME = MException( msg.Identifier, getString(msg) );
        throwAsCaller(ME);        
    end

end % end of assertThat.
