 classdef   CalinskiHarabaszEvaluation  < clustering.evaluation.ClusterCriterion
 % CalinskiHarabaszEvaluation Evaluation based on CalinskiHarabasz
%     A CalinskiHarabaszEvaluation object computes the CalinskiHarabasz
%     criterion values which can be used for estimating the number of
%     clusters on data. It's created using the following syntax:
%
%     EVA = EVALCLUSTERS(X,CLUST,'CalinskiHarabasz')
%
%   CalinskiHarabaszEvaluation properties:
%      X                     - Data used to create this object.
%      NumObservations       - Number of observations in X.
%      OptimalK              - The optimal number of clusters suggested.
%      OptimalY              - The optimal clustering solution.
%      InspectedK            - List of the number of clusters inspected.
%      CriterionName         - Name of the criterion.
%      CriterionValues       - The CalinskiHarabasz values for each number of clusters.
%      ClusteringFunction    - Clustering algorithm used.
%      Missing               - A vector indicating which observations are missing.
%
%   CalinskiHarabaszEvaluation methods:
%      compact               - Compact this object
%      plot                  - Plot the CalinskiHarabasz values
%
%   See also EVALCLUSTERS, clustering.evaluation.GapEvaluation,
%            clustering.evaluation.SilhouetteEvaluation,
%            clustering.evaluation.DaviesBouldinEvaluation.

%   Copyright 2013 The MathWorks, Inc.

  properties(GetAccess=protected, SetAccess=protected) 
      GlobalMean =[];
 end

  methods
      function hout = plot(this)
        h = plot@clustering.evaluation.ClusterCriterion(this);
        ylabel(getString(message('stats:clustering:evaluation:CalinskiHarabaszEvaluation:PlotYlabel')));
        if nargout > 0
            hout = h;
        end
      end
  end
  
  methods (Hidden)
    function   this = CalinskiHarabaszEvaluation(X,Y,varargin)
      narginchk(2,inf);
      [basePnames, baseDflts ]= clustering.evaluation.ClusterCriterion.getAllArgs();
       
      klist = ...
      internal.stats.parseArgs(basePnames, baseDflts, varargin{:});
      
      this = this@clustering.evaluation.ClusterCriterion(X,Y,klist);
      this.CriterionName = 'CalinskiHarabasz';
      this.GlobalMean = mean(this.PrivX,1);
                
      %Need to convert soft score to hard clustering solutions
      if iscell(Y)
          Y= this.soft2hard(Y);
      end
      
      if isnumeric(Y) % Y is hard solution
          Y(this.Missing,:)=[]; %removing missing values in the clustering solutions
          for j=1:numel(this.InspectedK)
              NC = this.InspectedK(j);
              [centroids, Ni, SUMD] = getCandSUMD(this, Y(:,j));
              this.CriterionValues(j)= getCH(this,centroids, Ni, SUMD,NC);      
          end
          
          [optimalV,optimalKLoc] = max(this.CriterionValues);
          if ~isnan(optimalV)
              this.OptimalK=this.InspectedK(optimalKLoc);
          end
          
      elseif ischar(Y) && this.FunLoc==1% built-in kmeans
          optimalCritV = -inf;
          for j=1: numel(this.InspectedK)
              NC= this.InspectedK(j);
              %SUMD is the sum of squared Euclidean Distance
              [IDX,centroids,SUMD]=kmeans(this.PrivX,NC,'rep',5,'emptyaction','singleton');
              Ni= accumarray(IDX,ones(this.NumObservations,1));
              this.CriterionValues(j)= getCH(this,centroids, Ni, SUMD,NC);
              if  this.CriterionValues(j) > optimalCritV
                  optimalCritV = this.CriterionValues(j);
                  this.OptimalK=this.InspectedK(j);
                  this.OptimalY = IDX;
              end
              
          end
      else %all other clustering algorithms
          optimalCritV = -inf;
          for j=1: numel(this.InspectedK)
              NC= this.InspectedK(j);
              IDX  = this.evalFun(NC);
              
              if isa(Y,'function_handle')
                  IDX = checkFuncHandle(this,IDX,NC);
              end
              [centroids, Ni, SUMD] = getCandSUMD(this, IDX);
              this.CriterionValues(j)= getCH(this,centroids, Ni, SUMD,NC);
              if  this.CriterionValues(j) > optimalCritV
                  optimalCritV = this.CriterionValues(j);
                  this.OptimalK=this.InspectedK(j);
                  this.OptimalY =IDX;
              end
          end
      end
      
      if any(this.Missing) && ~isempty(this.OptimalY)
          this.OptimalY = internal.stats.insertnan(this.Missing,this.OptimalY);
      end
      
      this.PrivX = [];
  end
  
  end %end public methods
  
  methods (Access = protected)
  %------------------------
  function [centroids, counts, sumD] = getCandSUMD(this, index)
      %get centroids, number of observations and sum of Squared Euclidean distance for
      %each cluster based on cluster index
      %index doesn't need to be integers between 1:number of clusters
      p = size(this.PrivX,2);
      clusts = unique(index);

      num = length(clusts);
      centroids = NaN(num,p);
      counts = zeros(num,1);
      sumD = zeros(num,1);
      for i = 1:num
          members = (index == clusts(i));
          if any(members)
              counts(i) = sum(members);
              centroids(i,:) = sum(this.PrivX(members,:),1) / counts(i);
              sumD(i)= sum(pdist2(this.PrivX(members,:),centroids(i,:)).^2);          
          end
      end
  end
  
  function CH = getCH(this,centroids, Ni, SUMD,NC)
     
      SSW = sum(SUMD,1);
      SSB = (pdist2(centroids,this.GlobalMean)).^2;
    
      SSB = sum(Ni.*SSB);
      CH =(SSB/(NC-1))/(SSW/(this.NumObservations-NC));
  end
end
end