classdef   GapEvaluation  < clustering.evaluation.ClusterCriterion
%GapEvaluation Evaluation object based on the gap values
%   A GapEvaluation object computes the gap criterion values which can be used
%   for estimating the number of clusters on data. It's created using the
%   following syntax
%
%   EVA = EVALCLUSTERS(X,CLUST,'GAP')
%
%   GapEvaluation properties:
%      X                      - Data used to create this object.
%      Distance               - The distance metric.
%      NumObservations        - Number of observations in X.
%      OptimalK               - The optimal number of clusters suggested.
%      OptimalY               - The optimal clustering solution.
%      InspectedK             - List of the number of clusters inspected.
%      CriterionName          - Name of the criterion.
%      CriterionValues        - The gap values for each number of clusters.
%      ClusteringFunction     - Clustering algorithm used.
%      Missing                - A vector indicating which observations are missing.
%      B                      - Number of reference data sets.
%      ReferenceDistribution  - Method used to generate reference data.
%      SearchMethod           - Method used to find the optimal clustering solution.
%      LogW                   - Natural logarithm of W on the input data.
%      ExpectedLogW           - Expectation of the natural logarithm of W on the reference data.
%      SE                     - Standard error of the expected logW for each number of clusters on the reference data.
%      StdLogW                - Standard deviation of the natural logarithm of W on the reference data.
%
%   GapEvaluation methods:
%     compact          - Compact this object.
%     plot             - Plot the gap values.
%
%   See also EVALCLUSTERS,clustering.evaluation.SilhouetteEvaluation
%            clustering.evaluation.CalinskiHarabaszEvaluation,
%            clustering.evaluation.DaviesBouldinEvaluation.

% References:
%    Tibshirani, R., Walther, G. and Hastie, T. (2001) Estimating the
%    number of data clusters via the Gap statistic. Journal of the Royal
%    Statistical Society B, 63, p411423


%   Copyright 2013 The MathWorks, Inc.
    
    properties(GetAccess=public,SetAccess=protected)
        %Distance The distance metric.
        %   The Distance property specifies the distance metric that used.
        %   It's a string representing a built-in distance measure, or a
        %   function handle.
        Distance = 'sqEuclidean';
        
        %B the number of reference data sets
        %   The B property is a integer representing the number of
        %   reference data sets used for computing gap criterion values.
        B = 100;
        
        %ReferenceDistribution The method used to generate the reference data
        %   The ReferenceDistribution property is a string how reference
        %   data are generated. It is 'uniform' or 'PCA'. The 'uniform'
        %   value means generating each reference feature uniformly over
        %   the range of each feature of the X property. 'PCA' means
        %   generating the reference feature from a uniform distribution
        %   over a box aligned with the principal components of the X
        %   property.
        ReferenceDistribution = 'PCA'
       
        %LogW Natural logarithm of W on the input data.
        %   The LogW property is a numeric vector containing the natural
        %   logarithm of W on the input data for each number of clusters in
        %   the InspectedK property. W is the sum of the within-cluster
        %   average pairwise distance, summed over all the clusters. If
        %   the distance is the squared Euclidean distance, then W is the
        %   sum of within-cluster sum of point-to-centroid squared
        %   Euclidean distance.
        LogW;
        
        %ExpectedLogW Expectation of the natural logarithm of W on the reference data.
        %   The ExpectedLogW property is a numeric vector containing the
        %   expectation of the natural logarithm of W on the reference data
        %   for each number of clusters in the InspectedK property. W is
        %   the sum of the within-cluster average pairwise distance,
        %   summed over all the clusters. If the distance is the squared
        %   Euclidean distance, then W is the sum of within-cluster sum of
        %   point-to-centroid squared Euclidean distance.
        ExpectedLogW;
        
        %StdLogW Standard deviation of the natural logarithm of W on the reference data.
        %   The StdLogW property is a numeric vector containing the
        %   standard deviation of the natural logarithm of W on reference
        %   data for each number of clusters. W is the sum of the
        %   within-cluster average pairwise distance, summed over all the
        %   clusters. If the distance is the squared Euclidean distance,
        %   then W is the sum of within-cluster sum of point-to-centroid
        %   squared Euclidean distance.
        StdLogW;
        
        %SE Standard error of the natural logarithm of W on the reference data.
        %   The SE property is a numeric vector containing the
        %   standard error of the natural logarithm of W on reference
        %   data for each number of clusters in the InspectedK property. W
        %   is the sum of the within-cluster average pairwise distance,
        %   summed over all the clusters. If the distance is the squared
        %   Euclidean distance, then W is the sum of within-cluster sum of
        %   point-to-centroid squared Euclidean distance.
        SE; 
        
        %SearchMethod Method used to find the optimal clustering solution.
        %   The SearchMethod property is a string indicating how the
        %   optimal number of clusters are estimated. It's either
        %   'firstmaxse' or 'globalmaxse'.
        SearchMethod; 
    end
    
    properties(GetAccess=protected,SetAccess=protected)
        VTransform % the transfer of V, [U,S,V]=svd(X0)
        minX2;  % The minimal value of X2, X2= X0*V; it will be empty if PCA is used
        maxX2;  % The maximal value of X2, X2= X0*V; it will be empty if PCA is used
        refDistLoc %1:PCA reference distribution, 2: uniform reference distribution
        SearchMethodLoc %1-FirstMaxSe 2 -GlobalMaxSE
        
        NXCols; % The number of columns in X
        minX;
        maxX;
        needSort = false;
    end
    
    methods (Access = private)

        function  this = getRefStats(this,refLogW)
              %after LogW and refLogW are computed
            this.ExpectedLogW = mean(refLogW,1);
            this.StdLogW = std(refLogW,1,1);
            %The standard error
            this.SE = this.StdLogW*sqrt(1+1/this.B);
        end
        
        %generate reference data
        function Xref = getReferenceData(this)
            if this.refDistLoc == 1%PCA reference
                Xref = getUniformRef(this.minX2,this.maxX2,...
                    this.NumObservations,this.NXCols);
                Xref = Xref*this.VTransform;
                %The mean of Xref has been shifted, but it's OK because
                %shifting the mean doesn't affect the separation of the
                %data.
               
            else % uniform  reference
                Xref = getUniformRef(this.minX,this.maxX, this.NumObservations,...
                    this.NXCols);
            end
        end
        
        function this = getSVD(this)
            meanX=mean(this.PrivX);
            X0= bsxfun(@minus,this.PrivX,meanX);
            [~,~,V] = svd(X0);
            X2=X0*V;
            this.minX2 = min(X2,[],1);
            this.maxX2 = max(X2,[],1);
            this.VTransform = V';
        end
    end
    
    methods
        function hout = plot(this)
        %PLOT  Plot the gap values.
        %  PLOT(S) plots the gap values with one standard error bars versus
        %  the number of clusters.
        %
        %  H = PLOT(E) returns the handle to the line in the plot.
        
            if this.needSort
                [sortedKList, IDX] = sort(this.InspectedK);
                h = errorbar(sortedKList,this.CriterionValues(IDX),this.SE(IDX),'-ob');
            else
                h = errorbar(this.InspectedK,this.CriterionValues,this.SE,'-ob');
            end
            if ~isnan(this.OptimalK)
                optimalVal = this.CriterionValues(this.InspectedK == this.OptimalK);
                line(this.OptimalK,optimalVal,'Marker','.','MarkerSize',10);
            end
            xlabel(getString(message('stats:clustering:evaluation:ClusterCriterion:PlotXlabel')));
            ylabel(getString(message('stats:clustering:evaluation:GapEvaluation:Ylabel')));
            if nargout > 0
                hout = h;
            end
        end
    end
    
   methods (Hidden)
        function   this = GapEvaluation(X,Y,varargin)
            narginchk(2,inf);
            [basePnames, baseDflts ]= clustering.evaluation.ClusterCriterion.getAllArgs();
            
            [klist,~,args] = ...
                internal.stats.parseArgs(basePnames, baseDflts, varargin{:});
                           
            if isempty(klist)
                error(message('stats:clustering:evaluation:GapEvaluation:EmptyKList'));
            end
            
            this = this@clustering.evaluation.ClusterCriterion(X,Y,klist);
            this.CriterionName = 'Gap';
            p = numel(this.InspectedK);
            
            [sortedKlist,sortIdx]= sort( this.InspectedK);
            
            if any(sortedKlist ~= this.InspectedK)
                this.needSort = true;
                this.InspectedK = sortedKlist;
            end
            
            if p ~= 1 && all( diff(this.InspectedK) == 1)
                defaultSearch ='FirstMaxSE';
            else
                defaultSearch ='GlobalMaxSE';
            end
            
            pnames = {'Distance'     'ReferenceDistribution'  'B' 'SearchMethod'};
            dflts =  {'sqeuclidean'  'PCA'                    100  defaultSearch };
            
            [distance, refDist,BNum, searchMethod]=...
                internal.stats.parseArgs(pnames, dflts, args{:});
            
            %Sanity check for distance
            %Distance must be a string or a function handle
           
            distList = {'sqEuclidean' 'Euclidean' 'cityblock'  'cosine' 'correlation' 'Hamming', 'Jaccard'};
            if ischar(distance)
                [this.Distance,distLoc] = internal.stats.getParamVal(...
                    distance,distList,'distance');
            elseif isa(distance,'function_handle')
                distLoc = 0;
                this.Distance = distance;
            else
                error(message('stats:clustering:evaluation:GapEvaluation:BadDist'));
            end
           
            if distLoc ~= 1 && ischar(Y)
                %if Y is builtin function and the distance is not sqEuclidean
                %distance
                if this.FunLoc == 1
                    this.ClustFuncHandle = getKmeansFunc(this.Distance);
                    %The above line is te same as the following code. 
                    %we use the above line to make sure the onCleanup
                    %object deconstructor will be called when needed.
                   %this.ClustFuncHandle = ...
                    %   @(X,NC)(kmeans(X,NC,'rep',5,'EmptyAction','singleton','distance',this.Distance));
                elseif this.FunLoc == 2 %linkage
                    if distLoc ~= 2 %the distance is neither 'euclidean' nor 'sqeuclidean'
                       this.ClustFuncHandle = getLinkageFunc(this.Distance);
                       %The above line is te same as the following code. 
                       %we use the above line to make sure the onCleanup
                       %object deconstructor will be called when needed.
                      % this.ClustFuncHandle = ...
                     %   @(X,NC)(clusterdata(X,'linkage','ave','maxclust',NC,'distance',this.Distance));
                    end
                   
                elseif this.FunLoc == 3 %gmdistribution
                    error(message('stats:clustering:evaluation:GapEvaluation:InvalidDistForGM'));
                end
            end
            
            %Sanity check for ReferenceDistribution
            [this.ReferenceDistribution,this.refDistLoc]=internal.stats.getParamVal(...
                refDist,{'PCA','Uniform'},'ReferenceDistribution');
            
            if this.refDistLoc == 1; %PCA reference distribution
                this = getSVD(this);
            else
                this.maxX = max(this.PrivX,[],1);
                this.minX = min(this.PrivX,[],1);
            end
            
            if ~isscalar(BNum) ||~isnumeric(BNum) || BNum <= 0 || round(BNum) ~= BNum ||...
                    isinf(BNum) ||  isnan(BNum)
                error(message('stats:clustering:evaluation:GapEvaluation:InvalidB'));
                
            end
            this.B = BNum;
            
            [this.SearchMethod,this.SearchMethodLoc] = ....
                internal.stats.getParamVal(...
                searchMethod,{'FirstMaxSE','GlobalMaxSE'},'SearchMethod');
            
            this.NXCols= size(X,2);
            
            this.ExpectedLogW = zeros(1,p);
            this.LogW = zeros(1,p);
            this.StdLogW = zeros(1,p);
            refLogW = zeros(BNum,p);
           
                      
            if ischar(Y) && this.FunLoc==1 && distLoc == 1
                %In the case kmeans with squared Euclidean distance
                
                warnCell={ 'stats:kmeans:EmptyClusterInBatchUpdate',...
                    'stats:kmeans:FailedToConvergeRep',...
                    'stats:kmeans:EmptyClusterRep'};
                
                for whichID = 1:numel(warnCell)
                    status(whichID) = warning('off', warnCell{whichID});
                end
                restoreWS = onCleanup(@() warning(status));
                                
                for b = 1:this.B
                    %generate reference data B times
                    XRef = getReferenceData(this);
                    for j= 1:p
                        NC = this.InspectedK(j);
                        [~, ~,SUMD_ref]=kmeans(XRef,NC,'rep',5,'empty','singleton');
                        refLogW(b,j)=log(sum(SUMD_ref,1));
                    end
                end
                this = getRefStats(this,refLogW);
                warning(status);
                if this.SearchMethodLoc == 1 %FirstMaxSE
           
                    for j = p:-1:1
                        NC = this.InspectedK(j);
                        [IDX, ~, SUMD]=kmeans(this.PrivX,NC,'rep',5,'empty','singleton');
                        this.LogW(j)= log(sum(SUMD,1));
                        %compute gap value
                        this.CriterionValues(j) = this.ExpectedLogW(j) ...
                            - this.LogW(j);
                        if ( j == p || this.CriterionValues(j) >=...
                                (this.CriterionValues(j+1)-this.SE(j+1)))
                            this.OptimalK = NC;
                            this.OptimalY = IDX;
                        end
                   end
                else%GlobalMaxSE
                    maxGap= -inf;
                    th = -inf;
                    this.OptimalK = NaN;
                    for i=p:-1:1
                        NC = this.InspectedK(i);
                        [IDX, ~, SUMD]=kmeans(this.PrivX,NC,'rep',5,'empty','singleton');
                        this.LogW(i) = log(sum(SUMD,1));
                        this.CriterionValues(i) = this.ExpectedLogW(i) - this.LogW(i);
                        if this.CriterionValues(i) >= maxGap
                            maxGap = this.CriterionValues(i);
                            th = maxGap - this.SE(i);
                        end
                        if  this.CriterionValues(i) >= th
                            this.OptimalK = NC;
                            this.OptimalY = IDX;
                        end        
                    end
                end
                   
            else  % all other case except kmeans with Squared Euclidean distance
                warnCell={'stats:gmdistribution:FailedToConverge',...
                    'stats:gmdistribution:FailedToConvergeReps',...
                    'stats:gmdistribution:IllCondCov',...
                    'stats:kmeans:EmptyClusterInBatchUpdate',...
                    'stats:kmeans:EmptyCluster',...
                    'stats:kmeans:EmptyClusterRep',...
                    'stats:kmeans:FailedToConverge',...
                    'stats:kmeans:FailedToConvergeRep'};
               
                for whichID = 1:numel(warnCell)
                    status(whichID) = warning('off', warnCell{whichID});
                end
                restoreWS = onCleanup(@() warning(status));
                for b= 1 : this.B
                    %generate reference data
                    XRef = getReferenceData(this);
                    
                    for j= 1:p
                        NC= this.InspectedK(j);
                        IDX = this.evalFunXref(NC,XRef);
                        if isa(Y,'function_handle')
                            IDX = checkFuncHandle(this,IDX,NC);
                        end
                        
                        refLogW(b,j) = getLogW(XRef, IDX,this.Distance,distLoc==1);
                    end
                    this = getRefStats(this,refLogW);
                 end
                warning(status);
                if this.SearchMethodLoc == 1 %FirstMaxSE
                    for i=p:-1:1
                        NC = this.InspectedK(i);
                        IDX =this.evalFun(NC);
                        if isa(Y,'function_handle')
                            IDX = checkFuncHandle(this,IDX,NC);
                        end
                        this.LogW(i) = getLogW(this.PrivX, IDX,this.Distance,distLoc==1);
                        this.CriterionValues(i) = this.ExpectedLogW(i) - this.LogW(i);
                        if ( i == p ||this.CriterionValues(i) >=...
                                (this.CriterionValues(i+1)-this.SE(i+1)))
                            this.OptimalK = NC;
                            this.OptimalY = IDX;
                        end
                        
                    end
                else%GlobalMaxSE
                    maxGap= -inf;
                    this.OptimalK = NaN;
                    for i=p:-1:1
                        NC= this.InspectedK(i);
                        IDX =this.evalFun(NC);
                        if isa(Y,'function_handle')
                            IDX = checkFuncHandle(this,IDX,NC);
                        end
                        this.LogW(i) = getLogW(this.PrivX, IDX,this.Distance,distLoc==1);
                        this.CriterionValues(i) = this.ExpectedLogW(i) - this.LogW(i);
                        if this.CriterionValues(i)>= maxGap
                            %update maxGap and SE(maxGap)
                            maxGap =this.CriterionValues(i);
                            th = maxGap-this.SE(i);
                        end
                        if  this.CriterionValues(i) >= th
                            this.OptimalK = NC;
                            this.OptimalY = IDX;
                        end  
                    end
               end
            end
            if any(this.Missing) %add the rows corresponding to missing values
                this.OptimalY = internal.stats.insertnan(this.Missing,this.OptimalY);
            end
            %resort Klist and CriterionValues based on the original order
            if this.needSort
                this.InspectedK(sortIdx) = this.InspectedK;
                this.CriterionValues(sortIdx) = this.CriterionValues;
            end
         
        end %GAP
        
    end %public methods
    
    
    methods (Access = protected)
        function IDX = evalFunXref(this,NC,Xref)
            try
                
                IDX = this.ClustFuncHandle(Xref,NC);
                
            catch ME
                if strcmp('MATLAB:UndefinedFunction', ME.identifier) ...
                        && ~isempty(strfind(ME.message, func2str(this.ClustFuncHandle)))
                    error(message('stats:clustering:evaluation:ClusterCriterion:FunNotFound', func2str(this.ClustFuncHandle )));
                else
                    error(message('stats:clustering:evaluation:GapEvaluation:FunErrorOnRef',NC, func2str(this.ClustFuncHandle), ME.message));
                end
            end
            
        end
    end %protected methods
    
    


end%class

%------------------------

function  logW = getLogW(X, index,dist,isSqEuclidean)
% Get log(W), where W is explained in formula 2 in reference 
p = size(X,2);
clusts = unique(index); %
num = numel(clusts);
centroids = nan(num,p);
sumD = nan(num,1);

if isSqEuclidean
    %get sum of squared Euclidean distance of point-to-centroid distances
    %if the distance is the squared Euclidean distance
    for i = 1:num
        members = (index == clusts(i));
        if any(members)
            ni = sum(members);
            centroids(i,:) = sum(X(members,:),1) / ni;
            sumD(i)= sum(pdist2(X(members,:),centroids(i,:)).^2);
            
        end
    end
else %other distance rather than squared Euclidean distance
    %Get the sum of all pairwise distances divided by twice of the number
    %of points 
    for i = 1:num
        members = (index == clusts(i));
        if any(members)
            ni = sum(members);
            sumD(i) = sum(pdist(X(members,:),dist))/ni;
        end
    end
end

logW = log(sum(sumD));
end

%generate uniform data between on the interval from minX and maxX
function Xref = getUniformRef(minX,maxX,nXRows,nXCols)
 Xref = bsxfun(@plus,minX,bsxfun(@times,(maxX-minX),rand(nXRows,nXCols)));
end

function fh = getKmeansFunc(distMetric)
%fh = @(X,NC)(kmeans(X,NC,'rep',5,'EmptyAction','singleton','distance',this.Distance));
     fh = @nested;
     function out = nested(X, NC)
        out=kmeans(X,NC, 'rep',5,'EmptyAction','singleton','distance',distMetric);
      end
end

function fh = getLinkageFunc(distMetric)
% fh = @(X,NC)(clusterdata(X,'linkage','ave','maxclust',NC,'distance',this.Distance));
     fh = @nested;
     function out = nested(X, NC)
        out= clusterdata(X,'linkage','ave','maxclust',NC,'distance',distMetric);
      end
end