classdef   SilhouetteEvaluation  < clustering.evaluation.ClusterCriterion
%SilhouetteEvaluation Evaluation based on silhouette criterion
%   A SilhouetteEvaluation object computes the silhouette criterion values
%   which can be used for estimating the number of clusters on data. 
%   It's created using the following syntax:
%
%       EVA = EVALCLUSTERS(X,CLUST,'Silhouette')
%
%   SilhouetteEvaluation properties:
%      X                     - Data used to create this object.
%      Distance              - The distance metric used.
%      NumObservations       - Number of observations in X.
%      OptimalK              - The optimal number of clusters suggested.
%      OptimalY              - The optimal clustering solution.
%      InspectedK            - List of the number of clusters inspected.
%      CriterionName         - Name of the criterion.
%      CriterionValues       - The silhouette values for each number of clusters.
%      ClusteringFunction    - Clustering algorithm used.
%      Missing               - A vector indicating which observations are missing.
%      ClusterPriors         - The cluster priors.
%      ClusterSilhouettes    - The silhouette values for each cluster in each clustering solution.
%
%   SilhouetteEvaluation methods:
%      compact               -  Compact this object
%      plot                  -  Plot the criterion values
%
%   See also EVALCLUSTERS, clustering.evaluation.GapEvaluation,
%            clustering.evaluation.CalinskiHarabaszEvaluation,
%            clustering.evaluation.DaviesBouldinEvaluation.

%   References:
%     Kaufman L. and Rousseeuw, P.J. Finding Groups in Data: An
%     Introduction to Cluster Analysis, Wiley, 1990


%  Copyright 2013 The MathWorks, Inc.

    properties(GetAccess=public, SetAccess=protected)
        %Distance The distance metric used.
        %   The Distance property specifies the distance metric used.
        %   It's a string representing a built-in distance measure, a
        %   function handle, or a numeric distance matrix in the vector
        %   form created by PDIST.
        Distance = 'sqEuclidean';
        
        %ClusterPriors The cluster priors.
        %   The ClusterPriors is one of the strings 'empirical' or 'equal'
        %   indicating the prior probabilities for each class.
        ClusterPriors = 'empirical';
        
        %ClusterSilhouettes Silhouette values in each cluster.
        %   The ClusterSilhouettes property is a cell array of matrices.
        %   The number of cells is equal to the number of clustering
        %   solutions. Each cell contains a matrix containing the average
        %   silhouette value for each cluster in the corresponding
        %   clustering solution.
        ClusterSilhouettes = [];
         
    end
    
    methods
        function hout = plot(this)
            h = plot@clustering.evaluation.ClusterCriterion(this);
            ylabel(getString(message('stats:clustering:evaluation:SilhouetteEvaluation:Ylabel')));
            if nargout > 0
                hout = h;
            end
        end
         
    end
    
    methods (Hidden)
        function   this = SilhouetteEvaluation(X,Y,varargin)
            narginchk(2,inf);
            [basePnames, baseDflts ]= clustering.evaluation.ClusterCriterion.getAllArgs();
            
            [klist,~,args] = ...
                internal.stats.parseArgs(basePnames, baseDflts, varargin{:});
            
            this= this@clustering.evaluation.ClusterCriterion(X,Y,klist);
            this.CriterionName = 'Silhouette';
          
         
            % parse input arguments for Silhouette
            pnames = {'Distance'     'ClusterPriors' };
            dflts =  { 'sqeuclidean'  'empirical' };
            
            [distMetric, prior] =...
                internal.stats.parseArgs(pnames, dflts, args{:});
            
            %check will be performed in function ''silhouette''
            %this.Distance = distMetric;
            p = numel(this.InspectedK);
            [this.Distance, distLoc] = checkDistance(distMetric, size(this.X,1));
            if isnumeric(this.Distance) % distance vector
                %deal with missing values in the distance vector
                if any(this.Missing) || any(isnan(this.Distance))
                    this.Distance = squareform(this.Distance,'tomatrix');
                    this.Missing = this.Missing | any(isnan(this.Distance),2);
              
                    this.NumObservations=sum(~this.Missing,1);
                    this.PrivX = this.X(~this.Missing,:);

                    this.Distance=this.Distance(~this.Missing,~this.Missing);
                    this.Distance = squareform(this.Distance,'tovector');
                    if isnumeric(Y)
                        for j=1:p
                           this.InspectedK(j) = numel(unique(Y(~this.Missing,j))); 
                        end
                    end
                end
            end
            
            if iscell(Y)%if Y is soft clustering solutions, convert it hard solutions
               Y = this.soft2hard(Y);
            end
            %Sanity check for clusterPrior
            [this.ClusterPriors,priorLoc]=internal.stats.getParamVal(...
                prior,{'equal','empirical'},'ClusterPrior');
                
                 
            this.ClusterSilhouettes=cell(1,p);

            optimalCritV = -inf;
            %loop over on each clustering solution
            if isnumeric(Y) % Y is a matrix
                Y(this.Missing,:)=[];
                for j=1:p
                    NC = this.InspectedK(j);
                    %apply the specified distance metric.
                    %The sanity check on distance will not be done here.
                    silVals= silhouette(this.PrivX,Y(:,j),this.Distance);
                    this.ClusterSilhouettes{j}=accumarray(grp2idx(Y(:,j)), silVals,[NC,1],@mean);
                    if priorLoc == 1 %equal 
                        % each cluster makes the same contribution 
                        this.CriterionValues(j)= mean(this.ClusterSilhouettes{j});
                    else %empirical
                        this.CriterionValues(j)= mean(silVals);
                    end
                  
                end
               % don't fill OptimalY property when Y is clustering solutions
               [maxVal,optimalKLoc]= max(this.CriterionValues);
               if ~isnan(maxVal)
                   this.OptimalK = this.InspectedK(optimalKLoc);
               end
            elseif ischar(Y) %built-in function 
                
                if distLoc ~= 1 % distance is not squared Euclidean distance
                    %Apply the specified distance metrics on built-in
                    %functions
                    if this.FunLoc == 1
                        %For kmeans, Distance must one of the string in distList
                        this.ClustFuncHandle = ...
                            @(X,NC)(kmeans(X,NC,'rep',5,'distance',this.Distance,'emptyaction','singleton'));
                    elseif this.FunLoc == 2 %linkage
                        %For linkage, distance must be one of the string in
                        %distList or a function handle or a distance matrix
                        if isnumeric(this.Distance) %distance is a distance matrix
                            this.ClustFuncHandle = ...
                                @(~,NC)(cluster(linkage(this.Distance,'ave'),'maxclust',NC));
                        elseif distLoc ~= 2% 'euclidean'
                               %the distance is neither 'euclidean' nor 'sqeuclidean'
                          this.ClustFuncHandle = ...
                                @(X,NC)(clusterdata(X,'linkage','ave','maxclust',NC,'distance',this.Distance));
                        end
                    elseif this.FunLoc == 3
                        %For gmdistribution, it must be sqEuclidean distance
                        error(message('stats:clustering:evaluation:SilhouetteEvaluation:BadDistForGM'));
                    end
                end % end distLoc ~= 1
              
              for j= 1:numel(this.InspectedK)
                  NC = this.InspectedK(j);
                  IDX  = this.evalFun(NC); 
                  
                  silVals = silhouette(this.PrivX,IDX, this.Distance);
                  this.ClusterSilhouettes{j}=accumarray(IDX, silVals,[NC,1],@mean);
                  if priorLoc == 1 %equal
                      this.CriterionValues(j)=mean(this.ClusterSilhouettes{j});
                  else % empirical prior
                      this.CriterionValues(j)=mean(silVals);
                  end
                  %need to find optimalY, 
                  if  this.CriterionValues(j) > optimalCritV
                      optimalCritV = this.CriterionValues(j);
                      this.OptimalK = this.InspectedK(j);
                      this.OptimalY =IDX;
                  end
              end
            else % must be a function handle
                %Check whether the requested distance is valid before we
                %call clustering algorithms
                 for j= 1:numel(this.InspectedK)
                    NC = this.InspectedK(j);
                    IDX  = this.evalFun(NC);
                    IDX = checkFuncHandle(this,IDX,NC);
                    silVals = silhouette(this.PrivX,IDX, this.Distance );
                    
                    this.ClusterSilhouettes{j}=accumarray(grp2idx(IDX), silVals,[NC,1],@mean);
                    if priorLoc == 1
                        this.CriterionValues(j)=mean(this.ClusterSilhouettes{j});
                    else
                        this.CriterionValues(j)=mean(silVals);
                    end
                    if  this.CriterionValues(j) > optimalCritV
                        optimalCritV = this.CriterionValues(j);
                        this.OptimalK = this.InspectedK(j);
                        this.OptimalY =IDX;
                    end
                end
                
            end
            
            if any(this.Missing) && ~isempty(this.OptimalY)
                this.OptimalY = internal.stats.insertnan(this.Missing,this.OptimalY);
            end
            
            this.PrivX = [];
        end
    end %end public methods
end

function [distance,distLoc] = checkDistance(distance,nIn)
if ischar(distance)
    distList = {'sqEuclidean' 'Euclidean' 'cityblock'  'cosine' 'correlation' 'Hamming', 'Jaccard'};
   [distance,distLoc] = internal.stats.getParamVal(distance,distList,'distance');
elseif isnumeric(distance)
    
    if (size(distance,1) == 1) && (size(distance,2) == .5*nIn*(nIn-1))
        if any(distance < 0)
            error(message('stats:clustering:evaluation:SilhouetteEvaluation:NegDistanceValues'));
        end
    else
        error(message('stats:silhouette:DistanceMatrixNotUpperTri'));
    end
    distLoc = 0;
elseif isa(distance,'function_handle')
   
    distLoc = 0;
else
    error(message('stats:clustering:evaluation:SilhouetteEvaluation:BadDist'));
end
end
