function c = nancov(x,varargin)
%MATLAB Code Generation Library Function

%   Limitations:
%   1. If the input is variably sized and is [] at run-time, returns []
%      rather than NaN.

%   Copyright 1984-2013 The MathWorks, Inc.
%#codegen

coder.internal.assert(nargin >= 1,'stats:nancov:NotEnoughInputs');
coder.internal.assert(nargin <= 4,'stats:nancov:TooManyArgs');
coder.internal.prefer_const(varargin);
[xycase,domle,dopairwise,domleIsDefault] = parseargs(varargin{:});
if coder.internal.const(xycase)
    y = varargin{1};
    coder.internal.errorIf(domleIsDefault && ...
        isscalar(y) && isnumeric(y) && (y(1) == 0 || y(1) == 1), ...
        'Coder:toolbox:CovAmbiguousMLEArg');
    coder.internal.assert(numel(x) == numel(y),'stats:nancov:XYmismatch');
    c = local_cov([x(:),y(:)],domle,dopairwise);
elseif isrow(x)
    c = local_cov(x(:),domle,dopairwise);
else
    c = local_cov(x,domle,dopairwise);
end

%--------------------------------------------------------------------------

function [xycase,domle,dopairwise,domleIsDefault] = parseargs(varargin)
coder.internal.prefer_const(varargin);
coder.inline('always');
% Initialize outputs to default values.
xycase = false;
domle = false;
dopairwise = false;
domleIsDefault = true;
% Quick return if there are no inputs.
if nargin > 0
    % Should we ignore NaNs by complete rows or pairwise?
    v1 = varargin{end};
    if ischar(v1)
        len = length(v1);
        dopairwise = strncmpi(v1,'pairwise',len);
        coder.internal.assert(dopairwise || ...
            strncmpi(v1,'complete',len), ...
            'stats:nancov:InvalidArg',v1);
        n1 = nargin - 1;
    else
        n1 = nargin;
    end
    if n1 > 0
        % Should we use the mle (divide by N) or unbiased estimate (N-1)?
        v2 = varargin{n1};
        if eml_is_const(v2) && isscalar(v2) && isnumeric(v2) && ...
                (v2 == 0 || v2 == 1)
            domle = (v2 == 1);
            n2 = n1 - 1;
            domleIsDefault = false;
        else
            n2 = n1;
        end
        if n2 > 0
            coder.internal.errorIf(n2 > 1,'stats:nancov:TooManyArgs');
            xycase = true;
        end
    end
end

%--------------------------------------------------------------------------

function c = local_cov(x,domle,dopairwise)
coder.internal.prefer_const(domle,dopairwise);
coder.internal.assert(ismatrix(x),'stats:nancov:InputDim');
m = coder.internal.indexInt(size(x,1));
n = coder.internal.indexInt(size(x,2));
if eml_is_const(size(x)) && isequal(size(x),[0,0])
    c = coder.internal.nan(class(x));
    return
end
c = zeros(n,'like',x);
if isempty(x)
    c(:) = coder.internal.nan(class(c));
    return
end
XZERO = zeros(1,'like',x);
IZERO = coder.internal.indexInt(0);
IONE = coder.internal.indexInt(1);
xnan = isnan(x);
anynans = vany(xnan);
if ~dopairwise || ~anynans
    if anynans
        % Remove rows with nans.
        nrows = IZERO;
        for k = 1:m
            keeprow = true;
            for j = 1:n
                if xnan(k,j)
                    keeprow = false;
                    break
                end
            end
            if keeprow
                nrows = nrows + 1;
                if nrows < k
                    for j = 1:n
                        x(nrows,j) = x(k,j);
                    end
                end
            end
        end
    else
        nrows = m;
    end
    XONE = XZERO + 1;
    LDA = coder.internal.indexInt(size(x,1));
    % if nrows == 1, c = zeros(n) is already the correct answer.
    if nrows == 0
        c(:) = coder.internal.nan(class(x));
    elseif nrows >= 2
        nx = cast(nrows,class(x));
        % Center the data.
        for j = 1:n
            % Calculate the mean.
            muj = XZERO;
            for i = 1:nrows
                muj = muj + x(i,j);
            end
            muj = muj/nx;
            % Center the column.
            for i = 1:nrows
                x(i,j) = x(i,j) - muj;
            end
        end
        if domle
            alpha = XONE/nx;
        else
            alpha = XONE/(nx - 1);
        end
        % c = xc' * xc / denom;
        % c = alpha*x(1:nrows,:)'*x(1:nrows,:);
        c(:) = 0;
        c = eml_xgemm('C','N',n,n,nrows, ...
            alpha,x,IONE,LDA,x,IONE,LDA,XZERO,c,IONE,n);
    end
else % pairwise with some NaNs
    for j = 1:n
        for i = 1:j
            mui = XZERO;
            muj = XZERO;
            nrows = IZERO;
            for k = 1:m
                if ~xnan(k,i) && ~xnan(k,j)
                    nrows = nrows + 1;
                    mui = mui + x(k,i);
                    muj = muj + x(k,j);
                end
            end
            if (domle && nrows <= 1) || (~domle && nrows == 0)
                c(i,j) = coder.internal.nan(class(x));
            else
                nx = cast(nrows,class(x));
                mui = mui/nx;
                muj = muj/nx;
                if domle
                    denom = nx;
                else
                    denom = nx - 1;
                end
                cij = XZERO;
                for k = 1:m
                    if ~xnan(k,i) && ~xnan(k,j)
                        cij = cij + (x(k,i) - mui)*(x(k,j) - muj);
                    end
                end
                cij = cij/denom;
                c(i,j) = cij;
            end
            if i < j
                c(j,i) = c(i,j);
            end
        end
    end
end

%--------------------------------------------------------------------------

function p = vany(x)
% Efficiently evaluate p = any(x(:)).
for k = 1:numel(x)
    if x(k)
        p = true;
        return
    end
end
p = false;

%--------------------------------------------------------------------------
